
#include <linux/kobject.h>
#include <linux/string.h>
#include <linux/sysfs.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/irq.h>
//#include <mach/gpio.h>
#include <linux/timer.h>
#include <linux/gpio.h>
#define MAX_ERROR_EVENT	16
#define MAX_INFOR_EVENT	26
#define MAX_EVENT_NAME_LEN  26
#define ISO_GPIO_BASE_NUMBER  60
#define TIMER_INTERVAL   (jiffies + HZ/2)
#define ERROR_EVENTS_SUPPRESS_INFOR_EVENT
typedef struct {
	char	 cName[MAX_EVENT_NAME_LEN];
	uint32_t  blink_time;
	uint32_t  blink_time_count;
	uint32_t  blink_on_time;
	uint32_t  blink_on_time_count;
	uint32_t  blink_off_time;
	uint32_t  blink_off_time_count;
	uint32_t  gpio_num;
	uint32_t  act;
	uint32_t  priority;
	uint32_t  enable;
}led_events_vars_t;


static ssize_t event_show(struct kobject *kobj, struct kobj_attribute *attr,
			char *buf);

static ssize_t event_store(struct kobject *kobj, struct kobj_attribute *attr,
			 const char *buf, size_t count);

static ssize_t power_show(struct kobject *kobj, struct kobj_attribute *attr,
			char *buf);

static ssize_t power_store(struct kobject *kobj, struct kobj_attribute *attr,
			 const char *buf, size_t count);

static struct timer_list LED_timer;

static bool fInit=false;
static led_events_vars_t error_event_vars[MAX_ERROR_EVENT];
static uint32_t error_event_vars_count=0;
static led_events_vars_t infor_event_vars[MAX_INFOR_EVENT];
static uint32_t infor_event_vars_count=0;
static led_events_vars_t power_event={"power", -1, -1, 2, 2, 2, 2, 73, 1, 99, true};
/*
static struct kobj_attribute foo_attribute =
	__ATTR(foo, 0666, foo_show, foo_store);
*/
static struct kobj_attribute error_event_kobj[MAX_ERROR_EVENT];
static struct kobj_attribute infor_event_kobj[MAX_INFOR_EVENT];
static struct kobj_attribute nas_events = __ATTR(events, 0666, event_show, event_store);
static struct kobj_attribute power_events_kobj = __ATTR(power, 0666, power_show, power_store);
static struct attribute *error_event_attrs[MAX_ERROR_EVENT];
static struct attribute *infor_event_attrs[MAX_INFOR_EVENT];
static struct kobject *nas_indicator_kobj;

static struct attribute_group error_event_group = {
	.name = "error_event",
	.attrs = error_event_attrs,
};

static struct attribute_group infor_event_group = {
	.name = "infor_event",
	.attrs = infor_event_attrs,
};

static void led_handler(unsigned long data)
{
	uint32_t attr_index;
	uint8_t own_power_gpio=true, power_led_on=false;

#ifdef ERROR_EVENTS_SUPPRESS_INFOR_EVENT 	
	bool     error_first=false;
#endif	
	if (fInit==false)
		return;

	for (attr_index=0 ; attr_index < error_event_vars_count ; attr_index++)
	{
		uint8_t own_gpio=true, led_on=false;
		if (error_event_vars[attr_index].enable==true)
		{
			uint32_t sub_attr_index;
			for (sub_attr_index=0 ; (sub_attr_index < attr_index)&&(own_gpio==true) ; sub_attr_index++)
			{
				if (error_event_vars[sub_attr_index].enable==true)
				{
					if (error_event_vars[sub_attr_index].gpio_num==error_event_vars[attr_index].gpio_num)
					{
						if (error_event_vars[sub_attr_index].priority<=error_event_vars[attr_index].priority)
						{
							own_gpio=false;
							break;
						}
					}
				}
			}
			for (sub_attr_index=attr_index+1 ; (sub_attr_index < error_event_vars_count)&&(own_gpio==true); sub_attr_index++)
			{
				if (error_event_vars[sub_attr_index].enable==true)
				{
					if (error_event_vars[sub_attr_index].gpio_num==error_event_vars[attr_index].gpio_num)
					{
						if (error_event_vars[sub_attr_index].priority<error_event_vars[attr_index].priority)
						{
							own_gpio=false;
							break;
						}
					}
				}
			}

			if (error_event_vars[attr_index].blink_on_time_count)
			{
				led_on=true;
				error_event_vars[attr_index].blink_on_time_count--;
			}
			else if (error_event_vars[attr_index].blink_off_time_count)
			{
				led_on=false;
				error_event_vars[attr_index].blink_off_time_count--;
			}
			if ((error_event_vars[attr_index].blink_on_time_count==0)&&(error_event_vars[attr_index].blink_off_time_count==0))
			{
				error_event_vars[attr_index].blink_on_time_count=error_event_vars[attr_index].blink_on_time;
				error_event_vars[attr_index].blink_off_time_count=error_event_vars[attr_index].blink_off_time;
			}

			if (error_event_vars[attr_index].blink_time!=-1)
			{
				if (error_event_vars[attr_index].blink_time_count==0)
				{
					led_on=false;
					error_event_vars[attr_index].enable=false;
				}
				else
				{
					error_event_vars[attr_index].blink_time_count--;
				}
			}
			if (own_gpio==true)
			{

#ifdef ERROR_EVENTS_SUPPRESS_INFOR_EVENT 	
				error_first=true;
#endif			
				own_power_gpio=false;
				if (led_on==true)
				{
					if (error_event_vars[attr_index].act==1)
						gpio_set_value(error_event_vars[attr_index].gpio_num, 1);
					else
						gpio_set_value(error_event_vars[attr_index].gpio_num, 0);
				}
				else
				{
					if (error_event_vars[attr_index].act==1)
						gpio_set_value(error_event_vars[attr_index].gpio_num, 0);
					else
						gpio_set_value(error_event_vars[attr_index].gpio_num, 1);
				}
			}
		}
	}

	for (attr_index=0 ; attr_index < infor_event_vars_count ; attr_index++)
	{
		uint8_t own_gpio=true, led_on=false;
		if (infor_event_vars[attr_index].enable==true)
		{
			uint32_t sub_attr_index;
			for (sub_attr_index=0 ; (sub_attr_index < error_event_vars_count)&&(own_gpio==true) ; sub_attr_index++)
			{
				if (error_event_vars[sub_attr_index].enable==true)
				{
					if (error_event_vars[sub_attr_index].gpio_num==infor_event_vars[attr_index].gpio_num)
					{
						own_gpio=false;
						break;
					}
				}
			}
			for (sub_attr_index=0 ; (sub_attr_index < attr_index)&&(own_gpio==true) ; sub_attr_index++)
			{
				if (infor_event_vars[sub_attr_index].enable==true)
				{
					if (infor_event_vars[sub_attr_index].gpio_num==infor_event_vars[attr_index].gpio_num)
					{
						if (infor_event_vars[sub_attr_index].priority<=infor_event_vars[attr_index].priority)
						{
							own_gpio=false;
							break;
						}
					}
				}
			}
			for (sub_attr_index=attr_index+1 ; (sub_attr_index < infor_event_vars_count)&&(own_gpio==true) ; sub_attr_index++)
			{
				if (infor_event_vars[sub_attr_index].enable==true)
				{
					if (infor_event_vars[sub_attr_index].gpio_num==infor_event_vars[attr_index].gpio_num)
					{
						if (infor_event_vars[sub_attr_index].priority<infor_event_vars[attr_index].priority)
						{
							own_gpio=false;
							break;
						}
					}
				}
			}


			if (infor_event_vars[attr_index].blink_on_time_count)
			{
				led_on=true;
				infor_event_vars[attr_index].blink_on_time_count--;
			}
			else if (infor_event_vars[attr_index].blink_off_time_count)
			{
				led_on=false;
				infor_event_vars[attr_index].blink_off_time_count--;
			}
			if ((infor_event_vars[attr_index].blink_on_time_count==0)&&(infor_event_vars[attr_index].blink_off_time_count==0))
			{
				infor_event_vars[attr_index].blink_on_time_count=infor_event_vars[attr_index].blink_on_time;
				infor_event_vars[attr_index].blink_off_time_count=infor_event_vars[attr_index].blink_off_time;
			}

			if (infor_event_vars[attr_index].blink_time!=-1)
			{
				if (infor_event_vars[attr_index].blink_time_count==0)
				{
					led_on=false;
					infor_event_vars[attr_index].enable=false;
				}
				else
				{
					infor_event_vars[attr_index].blink_time_count--;
				}
			}

			if (own_gpio==true)
			{
#ifdef ERROR_EVENTS_SUPPRESS_INFOR_EVENT 	
				if (error_first==true)
					led_on=false;
#endif
				own_power_gpio=false;
				if (led_on==true)
				{
					if (infor_event_vars[attr_index].act==1)
						gpio_set_value(infor_event_vars[attr_index].gpio_num, 1);
					else
						gpio_set_value(infor_event_vars[attr_index].gpio_num, 0);
				}
				else
				{
					if (infor_event_vars[attr_index].act==1)
						gpio_set_value(infor_event_vars[attr_index].gpio_num, 0);
					else
						gpio_set_value(infor_event_vars[attr_index].gpio_num, 1);
				}
			}
		}
	}

	if (power_event.enable==true)
	{
		if (power_event.blink_on_time_count)
		{
			power_led_on=true;
			power_event.blink_on_time_count--;
		}
		else if (power_event.blink_off_time_count)
		{
			power_led_on=false;
			power_event.blink_off_time_count--;
		}
		if ((power_event.blink_on_time_count==0)&&(power_event.blink_off_time_count==0))
		{
			power_event.blink_on_time_count=power_event.blink_on_time;
			power_event.blink_off_time_count=power_event.blink_off_time;
		}

		if (power_event.blink_time!=-1)
		{
			if (power_event.blink_time_count==0)
			{
				power_led_on=false;
				power_event.enable=false;
			}
			else
			{
				power_event.blink_time_count--;
			}
		}
		if (own_power_gpio==true)
		{
			if (power_led_on==true)
			{
				if (power_event.act==1)
					gpio_set_value(power_event.gpio_num, 1);
				else
					gpio_set_value(power_event.gpio_num, 0);
			}
			else
			{
				if (power_event.act==1)
					gpio_set_value(power_event.gpio_num, 0);
				else
					gpio_set_value(power_event.gpio_num, 1);
			}
		}
		else
		{
			if (power_event.act==1)
				gpio_set_value(power_event.gpio_num, 0);
			else
				gpio_set_value(power_event.gpio_num, 1);

		}
	}

	mod_timer(&LED_timer,TIMER_INTERVAL);
}

static uint32_t search_error_event(struct kobj_attribute *attr)
{
	uint32_t attr_index;
	for (attr_index=0; error_event_attrs[attr_index]!=NULL;attr_index++)
	{
		if (&error_event_kobj[attr_index]==attr)
			break;
	}

	if (attr_index >=error_event_vars_count)
	{
		return -1;
	}
	return attr_index;
}

static uint32_t search_infor_event(struct kobj_attribute *attr)
{
	uint32_t attr_index;
	for (attr_index=0; infor_event_attrs[attr_index]!=NULL;attr_index++)
	{
		if (&infor_event_kobj[attr_index]==attr)
			break;
	}

	if (attr_index >=infor_event_vars_count)
	{
		return -1;
	}
	return attr_index;
}
static ssize_t event_show(struct kobject *kobj, struct kobj_attribute *attr,
		char *buf)
{
	uint32_t attr_index, total=0;

	total+=sprintf(buf+total, "Error Event List:\n");
	total+=sprintf(buf+total, "\tname\t\ttotal\ton\toff\tgpio\tact\tpri\tstate\n");
	total+=sprintf(buf+total, "------------------------------------------------------------------------\n");
	for (attr_index=0 ; attr_index < error_event_vars_count ; attr_index++)
	{
		total+=sprintf(buf+total, "%20s\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",error_event_vars[attr_index].cName,error_event_vars[attr_index].blink_time, error_event_vars[attr_index].blink_on_time, error_event_vars[attr_index].blink_off_time,error_event_vars[attr_index].gpio_num, error_event_vars[attr_index].act, error_event_vars[attr_index].priority,  error_event_vars[attr_index].enable);
	}
	total+=sprintf(buf+total, "Information Event List:\n");
	total+=sprintf(buf+total, "\tname\t\ttotal\ton\toff\tgpio\tact\tpri\tstate\n");
	total+=sprintf(buf+total, "------------------------------------------------------------------------\n");
	for (attr_index=0 ; attr_index < infor_event_vars_count ; attr_index++)
	{
		total+=sprintf(buf+total, "%20s\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",infor_event_vars[attr_index].cName,infor_event_vars[attr_index].blink_time, infor_event_vars[attr_index].blink_on_time, infor_event_vars[attr_index].blink_off_time,infor_event_vars[attr_index].gpio_num, infor_event_vars[attr_index].act, infor_event_vars[attr_index].priority,  infor_event_vars[attr_index].enable);

	}

	return total;
}

static ssize_t event_store(struct kobject *kobj, struct kobj_attribute *attr,
		const char *buf, size_t count)
{
	return 0;
}

static ssize_t power_show(struct kobject *kobj, struct kobj_attribute *attr,
		char *buf)
{
	uint32_t total=0;

	total+=sprintf(buf+total, "\ttotal\ton\toff\tgpio\tact\tpri\tstate\n");
	total+=sprintf(buf+total, "-------------------------------------------------------------\n");
	total+=sprintf(buf+total, "\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",power_event.blink_time, power_event.blink_on_time, power_event.blink_off_time,power_event.gpio_num, power_event.act, power_event.priority,  power_event.enable);
#if 0
	total+=sprintf(buf+total, "Error event Name is %s\n", error_event_vars[attr_index].cName);
	total+=sprintf(buf+total, "Blink time %d sec, on is %d sec, off is %d sec (now %d)\n", error_event_vars[attr_index].blink_time, error_event_vars[attr_index].blink_on_time, error_event_vars[attr_index].blink_off_time,error_event_vars[attr_index].blink_time_count);
	total+=sprintf(buf+total, "GPIO Pin Number is %d\n", error_event_vars[attr_index].gpio_num);
	total+=sprintf(buf+total, "LED on is %s active\n", (error_event_vars[attr_index].act!=0)?"high":"low");
	total+=sprintf(buf+total, "Priority(hightest:0 ~ 99): %d\n", error_event_vars[attr_index].priority);
	total+=sprintf(buf+total, "This event is %s\n", (error_event_vars[attr_index].enable!=0)?"enabled":"disabled");
#endif
	return total;
}

static ssize_t power_store(struct kobject *kobj, struct kobj_attribute *attr,
		const char *buf, size_t count)
{
	uint32_t enable;
	/*reset event prarmeter, set enable bit to off first*/

	sscanf(buf, "%d %d %d %d %d %d %d", &power_event.blink_time, &power_event.blink_on_time, &power_event.blink_off_time,&power_event.gpio_num,&power_event.act,&power_event.priority,&enable);
	if (!gpio_is_valid(power_event.gpio_num))
	{
		enable=false;
		power_event.gpio_num=-2;/*-2 is for debug, means invalid gpio number*/
	}
	else
	{
		if (!gpio_request(power_event.gpio_num, NULL))
		{
			gpio_free(power_event.gpio_num);
			enable=false;
			power_event.gpio_num=-2;
		}
		else
		{
			/*set LED to off in initial state*/
			gpio_direction_output(power_event.gpio_num, 0);
			if (power_event.act==1)
				gpio_set_value(power_event.gpio_num, 0);
			else
				gpio_set_value(power_event.gpio_num, 1);
		}	
			power_event.blink_time_count=power_event.blink_time;
			power_event.blink_on_time_count=power_event.blink_on_time;
			power_event.blink_off_time_count=power_event.blink_off_time;
	}	

	power_event.enable=enable;
	return count;
}
static ssize_t error_event_show(struct kobject *kobj, struct kobj_attribute *attr,
		char *buf)
{
	uint32_t attr_index=search_error_event(attr);
	uint32_t total=0;

	if (attr_index==-1)
	{
		total+=sprintf(buf+total, "cannot find this event\n");
		return total;
	}
	total+=sprintf(buf+total, "\ttotal\ton\toff\tgpio\tact\tpri\tstate\n");
	total+=sprintf(buf+total, "-------------------------------------------------------------\n");
	total+=sprintf(buf+total, "\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",error_event_vars[attr_index].blink_time, error_event_vars[attr_index].blink_on_time, error_event_vars[attr_index].blink_off_time,error_event_vars[attr_index].gpio_num, error_event_vars[attr_index].act, error_event_vars[attr_index].priority,  error_event_vars[attr_index].enable);
#if 0
	total+=sprintf(buf+total, "Error event Name is %s\n", error_event_vars[attr_index].cName);
	total+=sprintf(buf+total, "Blink time %d sec, on is %d sec, off is %d sec (now %d)\n", error_event_vars[attr_index].blink_time, error_event_vars[attr_index].blink_on_time, error_event_vars[attr_index].blink_off_time,error_event_vars[attr_index].blink_time_count);
	total+=sprintf(buf+total, "GPIO Pin Number is %d\n", error_event_vars[attr_index].gpio_num);
	total+=sprintf(buf+total, "LED on is %s active\n", (error_event_vars[attr_index].act!=0)?"high":"low");
	total+=sprintf(buf+total, "Priority(hightest:0 ~ 99): %d\n", error_event_vars[attr_index].priority);
	total+=sprintf(buf+total, "This event is %s\n", (error_event_vars[attr_index].enable!=0)?"enabled":"disabled");
#endif
	return total;
}

static ssize_t error_event_store(struct kobject *kobj, struct kobj_attribute *attr,
		const char *buf, size_t count)
{
	uint32_t attr_index=search_error_event(attr);
	uint32_t enable;
	if (attr_index==-1)
	{
		return -1;
	}
	/*reset event prarmeter, set enable bit to off first*/
	error_event_vars[attr_index].enable=false;

	sscanf(buf, "%d %d %d %d %d %d %d", &error_event_vars[attr_index].blink_time, &error_event_vars[attr_index].blink_on_time, &error_event_vars[attr_index].blink_off_time,&error_event_vars[attr_index].gpio_num,&error_event_vars[attr_index].act,&error_event_vars[attr_index].priority,&enable);
	if (!gpio_is_valid(error_event_vars[attr_index].gpio_num))
	{
		enable=false;
		error_event_vars[attr_index].gpio_num=-2;/*-2 is for debug, means invalid gpio number*/
	}
	else
	{
		if (!gpio_request(error_event_vars[attr_index].gpio_num, NULL))
		{
			gpio_free(error_event_vars[attr_index].gpio_num);
			enable=false;
			error_event_vars[attr_index].gpio_num=-2;
		}
		else
		{
			uint32_t sub_attr_index;
			uint8_t own_gpio=true;
			for (sub_attr_index=0 ; sub_attr_index < error_event_vars_count ; sub_attr_index++)
			{
				if (error_event_vars[sub_attr_index].enable==true)
				{
					if (error_event_vars[sub_attr_index].gpio_num==error_event_vars[attr_index].gpio_num)
					{
						if (error_event_vars[sub_attr_index].priority<error_event_vars[attr_index].priority)
						{
							own_gpio=false;
							break;
						}
					}
				}
			}
			if (own_gpio==true)
			{
				/*set LED to off in initial state*/
				gpio_direction_output(error_event_vars[attr_index].gpio_num, 0);
				if (error_event_vars[attr_index].act==1)
					gpio_set_value(error_event_vars[attr_index].gpio_num, 0);
				else
					gpio_set_value(error_event_vars[attr_index].gpio_num, 1);
			}
			error_event_vars[attr_index].blink_time_count=error_event_vars[attr_index].blink_time;
			error_event_vars[attr_index].blink_on_time_count=error_event_vars[attr_index].blink_on_time;
			error_event_vars[attr_index].blink_off_time_count=error_event_vars[attr_index].blink_off_time;
		}	
	}	

	error_event_vars[attr_index].enable=enable;
	return count;
}

static ssize_t infor_event_show(struct kobject *kobj, struct kobj_attribute *attr,
		char *buf)
{
	uint32_t attr_index=search_infor_event(attr);
	uint32_t total=0;

	if (attr_index==-1)
	{
		total+=sprintf(buf+total, "cannot find this event\n");
		return total;
	}

	total+=sprintf(buf+total, "\ttotal\ton\toff\tgpio\tact\tpri\tstate\n");
	total+=sprintf(buf+total, "-------------------------------------------------------------\n");
	total+=sprintf(buf+total, "\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",infor_event_vars[attr_index].blink_time,infor_event_vars[attr_index].blink_on_time, infor_event_vars[attr_index].blink_off_time,infor_event_vars[attr_index].gpio_num, infor_event_vars[attr_index].act, infor_event_vars[attr_index].priority,  infor_event_vars[attr_index].enable);
#if 0
	total+=sprintf(buf+total, "Information event Name is %s\n", infor_event_vars[attr_index].cName);
	total+=sprintf(buf+total, "Blink time %d sec,  on %d sec, off %d sec (now %d)\n", infor_event_vars[attr_index].blink_time, infor_event_vars[attr_index].blink_on_time, infor_event_vars[attr_index].blink_off_time, infor_event_vars[attr_index].blink_time_count);
	total+=sprintf(buf+total, "GPIO Pin is Number %d\n", infor_event_vars[attr_index].gpio_num);
	total+=sprintf(buf+total, "LED on is %s active\n", (infor_event_vars[attr_index].act!=0)?"high":"low");
	total+=sprintf(buf+total, "Priority(hightest:0 ~ 99): %d\n", infor_event_vars[attr_index].priority);
	total+=sprintf(buf+total, "This event is %s\n", (infor_event_vars[attr_index].enable!=0)?"enabled":"disabled");
#endif
	return total;
}

static ssize_t infor_event_store(struct kobject *kobj, struct kobj_attribute *attr,
		const char *buf, size_t count)
{
	uint32_t attr_index=search_infor_event(attr);
	uint32_t enable;
	if (attr_index==-1)
	{
		return -1;
	}
	/*reset event prarmeter, set enable bit to off first*/
	infor_event_vars[attr_index].enable=false;

	sscanf(buf, "%d %d %d %d %d %d %d", &infor_event_vars[attr_index].blink_time, &infor_event_vars[attr_index].blink_on_time, &infor_event_vars[attr_index].blink_off_time,&infor_event_vars[attr_index].gpio_num,&infor_event_vars[attr_index].act,&infor_event_vars[attr_index].priority,&enable);
	if (!gpio_is_valid(infor_event_vars[attr_index].gpio_num))
	{
		enable=false;
		infor_event_vars[attr_index].gpio_num=-2;/*-2 is for debug, means invalid gpio number*/
	}
	else
	{
		if (!gpio_request(infor_event_vars[attr_index].gpio_num, NULL))
		{
			gpio_free(infor_event_vars[attr_index].gpio_num);
			enable=false;
			infor_event_vars[attr_index].gpio_num=-2;
		}
		else
		{
			uint32_t sub_attr_index;
			uint8_t own_gpio=true;
			for (sub_attr_index=0 ; sub_attr_index < infor_event_vars_count ; sub_attr_index++)
			{
				if (infor_event_vars[sub_attr_index].enable==true)
				{
					if (infor_event_vars[sub_attr_index].gpio_num==infor_event_vars[attr_index].gpio_num)
					{
						if (infor_event_vars[sub_attr_index].priority<infor_event_vars[attr_index].priority)
						{
							own_gpio=false;
							break;
						}
					}
				}
			}
			if (own_gpio==true)
			{
				/*set LED to off in initial state*/
				gpio_direction_output(infor_event_vars[attr_index].gpio_num, 0);
				if (infor_event_vars[attr_index].act==1)
					gpio_set_value(infor_event_vars[attr_index].gpio_num, 0);
				else
					gpio_set_value(infor_event_vars[attr_index].gpio_num, 1);
			}
			infor_event_vars[attr_index].blink_time_count=infor_event_vars[attr_index].blink_time;
			infor_event_vars[attr_index].blink_on_time_count=infor_event_vars[attr_index].blink_on_time;
			infor_event_vars[attr_index].blink_off_time_count=infor_event_vars[attr_index].blink_off_time;
		}	
	}	

	infor_event_vars[attr_index].enable=enable;
	return count;
}

static int __init nas_indicator_init(void)
{
	int retval, attr_index;

	if (fInit==false)
	{
		return -EPERM;
	}

	/*1. Get DTS event name*/
	for (attr_index=0;attr_index<error_event_vars_count;attr_index++)
	{
		error_event_kobj[attr_index].attr.name=error_event_vars[attr_index].cName;
		error_event_kobj[attr_index].attr.mode=0x666;
		error_event_kobj[attr_index].show=error_event_show;
		error_event_kobj[attr_index].store=error_event_store;
	}

	for (attr_index=0;attr_index<MAX_ERROR_EVENT;attr_index++)
	{
		if (attr_index<error_event_vars_count)
			error_event_attrs[attr_index]=&error_event_kobj[attr_index].attr;
		else
			error_event_attrs[attr_index]=NULL;
	}

	for (attr_index=0;attr_index<infor_event_vars_count;attr_index++)
	{
		infor_event_kobj[attr_index].attr.name=infor_event_vars[attr_index].cName;
		infor_event_kobj[attr_index].attr.mode=0x666;
		infor_event_kobj[attr_index].show=infor_event_show;
		infor_event_kobj[attr_index].store=infor_event_store;
	}

	for (attr_index=0;attr_index<MAX_INFOR_EVENT;attr_index++)
	{
		if (attr_index<infor_event_vars_count)
			infor_event_attrs[attr_index]=&infor_event_kobj[attr_index].attr;
		else
			infor_event_attrs[attr_index]=NULL;
	}

	nas_indicator_kobj = kobject_create_and_add("nas_indicator", kernel_kobj);
	if (!nas_indicator_kobj)
	{
		return -ENOMEM;
	}
	/* Create the files associated with this kobject */
	retval = sysfs_create_group(nas_indicator_kobj, &error_event_group);
	if (retval)
	{
		kobject_put(nas_indicator_kobj);
		return retval;
	}
	retval = sysfs_create_group(nas_indicator_kobj, &infor_event_group);
	if (retval)
	{
		kobject_put(nas_indicator_kobj);
		return retval;
	}

	retval = sysfs_create_file(nas_indicator_kobj, &nas_events.attr);
	if (retval)
	{
		kobject_put(nas_indicator_kobj);
		return retval;
	}
	retval = sysfs_create_file(nas_indicator_kobj, &power_events_kobj.attr);
	if (retval)
	{
		kobject_put(nas_indicator_kobj);
		return retval;
	}

	init_timer(&LED_timer);	
	LED_timer.function = led_handler;	
	LED_timer.data = 0L;	
	LED_timer.expires = TIMER_INTERVAL;	
	add_timer(&LED_timer);
	return 0;
}

static void __exit nas_indicator_exit(void)
{
	if (fInit==false)
	{
		return;
	}
	del_timer(&LED_timer);

	kobject_put(nas_indicator_kobj);
	return ;
}

module_init(nas_indicator_init);
module_exit(nas_indicator_exit);




static __init int nas_indicator_of_init(void)
{

	struct device_node  *dn       = NULL;
	dn = of_find_compatible_node(NULL, NULL, "Realtek,nas-led-indicator");
	if (!dn)
	{
		return -EPERM;
	}

	{/*Handle error event*/
		const char*      event_name= of_get_property(dn, "error_event", NULL);

		for (error_event_vars_count=0;error_event_vars_count <MAX_ERROR_EVENT; error_event_vars_count++)
		{
			/*event name*/
			if(of_property_read_string_index(dn, "error_event",error_event_vars_count , &event_name) < 0)
			{
				break;
			}
			snprintf(error_event_vars[error_event_vars_count].cName, MAX_EVENT_NAME_LEN-1, "%s",event_name);

			error_event_vars[error_event_vars_count].blink_time = -1;
			error_event_vars[error_event_vars_count].blink_time_count = -1;
			error_event_vars[error_event_vars_count].blink_on_time = -1;
			error_event_vars[error_event_vars_count].blink_on_time_count = -1;
			error_event_vars[error_event_vars_count].blink_off_time = -1;
			error_event_vars[error_event_vars_count].blink_off_time_count = -1;
			error_event_vars[error_event_vars_count].gpio_num = -1;
			error_event_vars[error_event_vars_count].priority=-1;
			error_event_vars[error_event_vars_count].act = -1;
			error_event_vars[error_event_vars_count].enable=false;
		}
	}
	{/*Handle infor event*/
		const char*      event_name= of_get_property(dn, "infor_event", NULL);

		for (infor_event_vars_count=0; infor_event_vars_count<MAX_INFOR_EVENT; infor_event_vars_count++)
		{
			/*event name*/
			if(of_property_read_string_index(dn, "infor_event", infor_event_vars_count, &event_name) < 0)
			{
				break;
			}
			snprintf(infor_event_vars[infor_event_vars_count].cName, MAX_EVENT_NAME_LEN-1, "%s",event_name);
			infor_event_vars[infor_event_vars_count].blink_time = -1;
			infor_event_vars[infor_event_vars_count].blink_time_count = -1;
			infor_event_vars[infor_event_vars_count].blink_on_time = -1;
			infor_event_vars[infor_event_vars_count].blink_on_time_count = -1;
			infor_event_vars[infor_event_vars_count].blink_off_time = -1;
			infor_event_vars[infor_event_vars_count].blink_off_time_count = -1;
			infor_event_vars[infor_event_vars_count].gpio_num = -1;
			infor_event_vars[infor_event_vars_count].priority=-1;
			infor_event_vars[infor_event_vars_count].act = -1;
			infor_event_vars[infor_event_vars_count].enable=false;
		}
	}
	fInit=true;
	return 0;
}
arch_initcall(nas_indicator_of_init);

