/* Copyright (C) 2015, Realtek, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/pci_ids.h>
#include <linux/crypto.h>
#include <linux/spinlock.h>
#include <crypto/algapi.h>
#include <crypto/aes.h>
#include <linux/dma-mapping.h>
#include <asm/page.h>
#include <linux/mm.h>
#include <linux/io.h>
#include <linux/delay.h>
#include "rtk_mcp.h"
#include "debug.h"
#define _mcp_map_single(p_data, len, dir)      dma_map_single(mcp_device, (void*) p_data, (size_t) len, dir)
#define _mcp_unmap_single(p_data, len, dir)    dma_unmap_single(mcp_device, (dma_addr_t) p_data, (size_t)  len, dir)
/* driver logic flags */
#define AES_IV_LENGTH  16
#define AES_KEY_LENGTH 16
#define AES_MIN_BLOCK_SIZE 16

#define AES_MODE_ECB 0
#define AES_MODE_CBC 1

#define AES_DIR_DECRYPT 0
#define AES_DIR_ENCRYPT 1

static struct device*   mcp_device;
struct rtk_aes_ctx {
	uint8_t key[AES_KEYSIZE_256];
	uint32_t keylen;
	union {
		struct crypto_blkcipher *blk;
		struct crypto_cipher *cip;
	} fallback;
};

struct rtk_aes_op {
	void *src;
	void *dst;
	uint32_t mode;
	uint32_t dir;
	uint32_t flags;
	int len;
	uint8_t iv[16];
	uint8_t key[AES_KEYSIZE_256];
	uint32_t keylen;
};
/* Static structures */
static spinlock_t lock;

extern int mcp_do_command(
		mcp_desc*               p_desc, 
		int                     n_desc
		);

static unsigned char writeSRAM(unsigned int id, unsigned int *data, unsigned int cnt)
{
	int i=0,y=0;
	for (i=0;i<cnt;i++)
	{
		//		dbg_info("%s %s %d==>data: %x",__FILE__, __func__,__LINE__,data[y]);
		SET_TP_KEYINFO_0(data[y++]);
		SET_TP_KEYINFO_1(data[y++]);
		SET_TP_KEY_CTRL((id+i) | 0x80);  // write 8 bytes
		nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();nop();
	}
	SET_TP_KEYINFO_0(0x0);
	SET_TP_KEYINFO_1(0x0);
	return 0;
}
/*====================================================================== 
 * Func : MCP_AES_DESC_INIT
 *
 * Desc : Init AES Descriptor
 *
 * Parm : pDesc      : Descriptor to be Load
 *        EnDe       : Encryption/Descryption
 *              0 for Decryption / 1 for Encryption
 *        Mode       : Operation Mode
 *        Key        : Key Value
 *        IV         : Initial Vector 
 *
 * Retn : S_OK /  S_FALSE 
 *======================================================================*/
static int mcp_make_desc(
		mcp_desc*               pDesc,
		uint32_t					keylen,	
		unsigned char           EnDe,
		unsigned char           Mode,
		unsigned char           *Key,
		unsigned char           *IV
		)
{            
	uint32_t aes_key[8];
	memset(pDesc, 0, sizeof(mcp_desc));

	switch (Mode)
	{
		case MCP_BCM_ECB:        
		case MCP_BCM_CBC:
		case MCP_BCM_CTR:                
			if (Key)                                        
			{      
				aes_key[0]  = (Key[ 0]<<24) + (Key[ 1]<<16) + (Key[ 2]<<8) + Key[ 3];  
				aes_key[1]  = (Key[ 4]<<24) + (Key[ 5]<<16) + (Key[ 6]<<8) + Key[ 7];  
				aes_key[2]  = (Key[ 8]<<24) + (Key[ 9]<<16) + (Key[10]<<8) + Key[11];  
				aes_key[3]  = (Key[12]<<24) + (Key[13]<<16) + (Key[14]<<8) + Key[15];  
				aes_key[4]  = (Key[16]<<24) + (Key[17]<<16) + (Key[18]<<8) + Key[19];  
				aes_key[5]  = (Key[20]<<24) + (Key[21]<<16) + (Key[22]<<8) + Key[23];  
				aes_key[6]  = (Key[24]<<24) + (Key[25]<<16) + (Key[26]<<8) + Key[27];  
				aes_key[7]  = (Key[28]<<24) + (Key[29]<<16) + (Key[30]<<8) + Key[31];  
			}
			if (IV)        
			{
				pDesc->iv[0]   = (IV[ 0]<<24) + (IV[ 1]<<16) + (IV[ 2]<<8) + IV[ 3];  
				pDesc->iv[1]   = (IV[ 4]<<24) + (IV[ 5]<<16) + (IV[ 6]<<8) + IV[ 7];  
				pDesc->iv[2]   = (IV[ 8]<<24) + (IV[ 9]<<16) + (IV[10]<<8) + IV[11];  
				pDesc->iv[3]   = (IV[12]<<24) + (IV[13]<<16) + (IV[14]<<8) + IV[15];
				SET_MCP_IV0(pDesc->iv[0]);
				SET_MCP_IV1(pDesc->iv[1]);
				SET_MCP_IV2(pDesc->iv[2]);
				SET_MCP_IV3(pDesc->iv[3]);
			}
			if (keylen==AES_KEYSIZE_128)
			{	
				writeSRAM(0, aes_key, 2);
				pDesc->flags = MARS_MCP_MODE(MCP_ALGO_AES) | MARS_MCP_IV_SEL(MCP_IV_SEL_REG) | MARS_MCP_BCM(Mode) | MARS_MCP_ENC(EnDe)  | MARS_MCP_KEY_SEL(MCP_KEY_SEL_CW);
			}
			else if (keylen==AES_KEYSIZE_192)
			{
				writeSRAM(0, aes_key, 3);
				pDesc->flags = MARS_MCP_MODE(MCP_ALGO_AES_192) | MARS_MCP_IV_SEL(MCP_IV_SEL_REG) | MARS_MCP_BCM(Mode) | MARS_MCP_ENC(EnDe) | MARS_MCP_KEY_SEL(MCP_KEY_SEL_CW);
			}
			else
			{	
				writeSRAM(0, aes_key, 4);
				pDesc->flags = MARS_MCP_MODE(MCP_ALGO_AES_256) | MARS_MCP_IV_SEL(MCP_IV_SEL_REG) | MARS_MCP_BCM(Mode) | MARS_MCP_ENC(EnDe) | MARS_MCP_KEY_SEL(MCP_KEY_SEL_CW);
			}
			return 0;                

		default:        
			mcp_warning("Init AES descriptor failed - invalid mode (%d)\n", Mode);
			return -1;
	}        
}





	static unsigned int
rtk_aes_crypt(struct rtk_aes_op *op)
{
	unsigned long len=0;
	mcp_desc    desc[2];
	unsigned char           Mode;
	int         ret;    
	dma_addr_t  addr1;
	dma_addr_t  addr2;
	void *src, *dst;
	uint8_t *IV=(uint8_t*)desc[0].iv;

	if (op->len == 0)
		return 0;

	/* If the source and destination is the same, then
	 * we need to turn on the coherent flags, otherwise
	 * we don't need to worry
	 */

	if (op->mode == AES_MODE_CBC) 
		Mode=MCP_BCM_CBC;
	else
		Mode=MCP_BCM_ECB;
	/* Start the critical section */

	if (0)
	{
		int i;
		dbg_info("iv begin");
		for (i=0;i<16&&op->iv!=NULL;i++)
		{
			dbg_info("0x%2x ", *(uint8_t*)(op->iv+i));
		}
		dbg_info("iv end");
	}
	if (0)
	{
		int i;
		dbg_info("key begin");
		for (i=0;i<16;i++)
		{
			dbg_info("0x%2x ", *(uint8_t*)(op->key+i));
		}
		dbg_info("key end");
	}
	mcp_make_desc(&desc[0],op->keylen,  op->dir, Mode, op->key, op->iv);
	if (0)
	{
		int i;
		dbg_info("src begin");
		for (i=0;i<20;i++)
		{
			dbg_info("0x%2x ", *(uint8_t*)(op->src+i));
		}
		dbg_info("src end");
	}
	len=op->len;
	src=op->src;
	dst=op->dst;	
	addr1 = _mcp_map_single(src, len, DMA_TO_DEVICE);    
	addr2 = _mcp_map_single(dst, len, DMA_FROM_DEVICE);    

	desc[0].data_in  = addr1;
	desc[0].data_out = addr2;
	desc[0].length   = op->len;    	
	ret = mcp_do_command(&desc[0], 1);   

	_mcp_unmap_single(addr1, len, DMA_TO_DEVICE);                    
	_mcp_unmap_single(addr2, len, DMA_FROM_DEVICE);
	if (0)
	{
		int i;
		dbg_info("dst begin (%ld)\n", len);
		for (i=0;i<20;i++)
		{
			dbg_info("0x%2x ", *((uint8_t*)op->dst+i));
		}
		dbg_info("dst end\n");
	}
	desc[0].iv[0]=GET_MCP_IV0();
	desc[0].iv[1]=GET_MCP_IV1();
	desc[0].iv[2]=GET_MCP_IV2();
	desc[0].iv[3]=GET_MCP_IV3();
	op->iv[3] = IV[0];
	op->iv[2] = IV[1];
	op->iv[1] = IV[2];
	op->iv[0] = IV[3];

	op->iv[7] = IV[4];
	op->iv[6] = IV[5];
	op->iv[5] = IV[6];
	op->iv[4] = IV[7];

	op->iv[11] = IV[8];
	op->iv[10] = IV[9];
	op->iv[9] = IV[10];
	op->iv[8] = IV[11];

	op->iv[15] = IV[12];
	op->iv[14] = IV[13];
	op->iv[13] = IV[14];
	op->iv[12] = IV[15];

	return len;
}


static int rtk_setkey_blk(struct crypto_tfm *tfm, const u8 *key,
		unsigned int len)
{

	struct rtk_aes_ctx *op = crypto_tfm_ctx(tfm);
	unsigned int ret;

	op->keylen = len;
	if ((len == AES_KEYSIZE_128)||(len == AES_KEYSIZE_192)||(len == AES_KEYSIZE_256)) {
		memcpy(op->key, key, len);
		return 0;
	}

	/*
	 * The requested key size is not supported by HW, do a fallback
	 */
	op->fallback.blk->base.crt_flags &= ~CRYPTO_TFM_REQ_MASK;
	op->fallback.blk->base.crt_flags |= (tfm->crt_flags & CRYPTO_TFM_REQ_MASK);

	ret = crypto_blkcipher_setkey(op->fallback.blk, key, len);
	if (ret) {
		tfm->crt_flags &= ~CRYPTO_TFM_RES_MASK;
		tfm->crt_flags |= (op->fallback.blk->base.crt_flags & CRYPTO_TFM_RES_MASK);
	}
	return ret;

}
#if 0
static int fallback_blk_dec(struct blkcipher_desc *desc,
		struct scatterlist *dst, struct scatterlist *src,
		unsigned int nbytes)
{
	unsigned int ret;
	struct crypto_blkcipher *tfm;
	struct rtk_aes_op *op = crypto_blkcipher_ctx(desc->tfm);

	tfm = desc->tfm;
	desc->tfm = op->fallback.blk;

	ret = crypto_blkcipher_decrypt_iv(desc, dst, src, nbytes);

	desc->tfm = tfm;
	return ret;
}
#endif
#if 0
static int fallback_blk_enc(struct blkcipher_desc *desc,
		struct scatterlist *dst, struct scatterlist *src,
		unsigned int nbytes)
{
	unsigned int ret;
	struct crypto_blkcipher *tfm;
	struct rtk_aes_ctx *op = crypto_blkcipher_ctx(desc->tfm);

	tfm = desc->tfm;
	desc->tfm = op->fallback.blk;

	ret = crypto_blkcipher_encrypt_iv(desc, dst, src, nbytes);

	desc->tfm = tfm;
	return ret;
}
#endif
	static int
rtk_cbc_decrypt(struct blkcipher_desc *desc,
		struct scatterlist *dst, struct scatterlist *src,
		unsigned int nbytes)
{
	unsigned long iflags;
	struct rtk_aes_ctx *ctx = crypto_blkcipher_ctx(desc->tfm);
	struct rtk_aes_op op;
	struct blkcipher_walk walk;
	int err, ret;

	blkcipher_walk_init(&walk, dst, src, nbytes);
	err = blkcipher_walk_virt(desc, &walk);
	memcpy(op.key, ctx->key, ctx->keylen);
	op.keylen=ctx->keylen;
	memcpy(op.iv, walk.iv,16);
	op.dir = AES_DIR_DECRYPT;
	op.mode = AES_MODE_CBC;
	spin_lock_irqsave(&lock, iflags);
	while ((nbytes = walk.nbytes)) {
		op.src = walk.src.virt.addr;
		op.dst = walk.dst.virt.addr;
		op.len = nbytes - (nbytes % AES_MIN_BLOCK_SIZE);
		ret = rtk_aes_crypt(&op);
		nbytes -= ret;
		memcpy(walk.iv,op.iv, 16);
		err = blkcipher_walk_done(desc, &walk, nbytes);
	}
	spin_unlock_irqrestore(&lock, iflags);
	return err;
}

	static int
rtk_cbc_encrypt(struct blkcipher_desc *desc,
		struct scatterlist *dst, struct scatterlist *src,
		unsigned int nbytes)
{
	unsigned long iflags;
	struct rtk_aes_ctx *ctx = crypto_blkcipher_ctx(desc->tfm);
	struct rtk_aes_op op;
	struct blkcipher_walk walk;
	int err, ret;

	blkcipher_walk_init(&walk, dst, src, nbytes);
	err = blkcipher_walk_virt(desc, &walk);
	memcpy(op.key, ctx->key, ctx->keylen);
	op.keylen=ctx->keylen;

	memcpy(op.iv, walk.iv,16);
	op.dir = AES_DIR_ENCRYPT;
	op.mode = AES_MODE_CBC;
	spin_lock_irqsave(&lock, iflags);
	while ((nbytes = walk.nbytes)) {
		op.src = walk.src.virt.addr;
		op.dst = walk.dst.virt.addr;
		op.len = nbytes - (nbytes % AES_MIN_BLOCK_SIZE);
		ret = rtk_aes_crypt(&op);
		nbytes -= ret;
		memcpy(walk.iv,op.iv, 16);
		err = blkcipher_walk_done(desc, &walk, nbytes);
	}
	spin_unlock_irqrestore(&lock, iflags);
	return err;
}

static int fallback_init_blk(struct crypto_tfm *tfm)
{
	const char *name = tfm->__crt_alg->cra_name;
	struct rtk_aes_ctx *op = crypto_tfm_ctx(tfm);

	op->fallback.blk = crypto_alloc_blkcipher(name, 0,
			CRYPTO_ALG_ASYNC | CRYPTO_ALG_NEED_FALLBACK);

	if (IS_ERR(op->fallback.blk)) {
		printk(KERN_ERR "Error allocating fallback algo %s\n", name);
		return PTR_ERR(op->fallback.blk);
	}
	return 0;
}

static void fallback_exit_blk(struct crypto_tfm *tfm)
{
	struct rtk_aes_ctx *op = crypto_tfm_ctx(tfm);

	crypto_free_blkcipher(op->fallback.blk);
	op->fallback.blk = NULL;
}

static struct crypto_alg rtk_cbc_alg = {
	.cra_name		=	"cbc(aes)",
	.cra_driver_name	=	"cbc-aes-rtk",
	.cra_priority		=	400,
	.cra_flags			=	CRYPTO_ALG_TYPE_BLKCIPHER |
		CRYPTO_ALG_KERN_DRIVER_ONLY |
		CRYPTO_ALG_NEED_FALLBACK,
	.cra_init			=	fallback_init_blk,
	.cra_exit			=	fallback_exit_blk,
	.cra_blocksize		=	AES_MIN_BLOCK_SIZE,
	.cra_ctxsize		=	sizeof(struct rtk_aes_ctx),
	.cra_alignmask		=	15,
	.cra_type			=	&crypto_blkcipher_type,
	.cra_module			=	THIS_MODULE,
	.cra_u				=	{
		.blkcipher	=	{
			.min_keysize	=	AES_MIN_KEY_SIZE,
			.max_keysize	=	AES_MAX_KEY_SIZE,
			.setkey			=	rtk_setkey_blk,
			.encrypt		=	rtk_cbc_encrypt,
			.decrypt		=	rtk_cbc_decrypt,
			.ivsize			=	AES_IV_LENGTH,
		}
	}
};

	static int
rtk_ecb_decrypt(struct blkcipher_desc *desc,
		struct scatterlist *dst, struct scatterlist *src,
		unsigned int nbytes)
{

	unsigned long iflags;
	struct rtk_aes_ctx *ctx = crypto_blkcipher_ctx(desc->tfm);
	struct rtk_aes_op op;
	struct blkcipher_walk walk;
	int err, ret;
	blkcipher_walk_init(&walk, dst, src, nbytes);
	err = blkcipher_walk_virt(desc, &walk);
	memcpy(op.key, ctx->key, ctx->keylen);
	op.keylen=ctx->keylen;

	op.dir = AES_DIR_DECRYPT;
	op.mode = AES_MODE_ECB;
	memcpy(op.iv, walk.iv,16);
	spin_lock_irqsave(&lock, iflags);
	while ((nbytes = walk.nbytes)) {
		op.src = walk.src.virt.addr;
		op.dst = walk.dst.virt.addr;
		op.len = nbytes - (nbytes % AES_MIN_BLOCK_SIZE);
		ret = rtk_aes_crypt(&op);
		nbytes -= ret;
		err = blkcipher_walk_done(desc, &walk, nbytes);
	}
	spin_unlock_irqrestore(&lock, iflags);

	return err;

}

	static int
rtk_ecb_encrypt(struct blkcipher_desc *desc,
		struct scatterlist *dst, struct scatterlist *src,
		unsigned int nbytes)
{
	unsigned long iflags;
	struct rtk_aes_ctx *ctx = crypto_blkcipher_ctx(desc->tfm);
	struct rtk_aes_op op;
	struct blkcipher_walk walk;
	int err, ret;

	blkcipher_walk_init(&walk, dst, src, nbytes);
	err = blkcipher_walk_virt(desc, &walk);
	memcpy(op.key, ctx->key, ctx->keylen);
	op.keylen=ctx->keylen;
	memcpy(op.iv, walk.iv,16);
	op.dir = AES_DIR_ENCRYPT;
	op.mode = AES_MODE_ECB;
	spin_lock_irqsave(&lock, iflags);
	while ((nbytes = walk.nbytes)) {
		op.src = walk.src.virt.addr;
		op.dst = walk.dst.virt.addr;
		op.len = nbytes - (nbytes % AES_MIN_BLOCK_SIZE);
		ret = rtk_aes_crypt(&op);
		nbytes -= ret;
		err =  blkcipher_walk_done(desc, &walk, nbytes);
	}
	spin_unlock_irqrestore(&lock, iflags);
	return err;

}

static struct crypto_alg rtk_ecb_alg = {
	.cra_name		=	"ecb(aes)",
	.cra_driver_name	=	"ecb-aes-rtk",
	.cra_priority		=	400,
	.cra_flags		=	CRYPTO_ALG_TYPE_BLKCIPHER |
		CRYPTO_ALG_KERN_DRIVER_ONLY |
		CRYPTO_ALG_NEED_FALLBACK,
	.cra_init		=	fallback_init_blk,
	.cra_exit		=	fallback_exit_blk,
	.cra_blocksize		=	AES_MIN_BLOCK_SIZE,
	.cra_ctxsize		=	sizeof(struct rtk_aes_ctx),
	.cra_alignmask		=	15,
	.cra_type		=	&crypto_blkcipher_type,
	.cra_module		=	THIS_MODULE,
	.cra_u			=	{
		.blkcipher		=	{
			.min_keysize	=	AES_MIN_KEY_SIZE,
			.max_keysize	=	AES_MAX_KEY_SIZE,
			.setkey		=	rtk_setkey_blk,
			.encrypt	=	rtk_ecb_encrypt,
			.decrypt	=	rtk_ecb_decrypt,
		}
	}
};

void rtk_aes_remove(void)
{
	crypto_unregister_alg(&rtk_ecb_alg);
	crypto_unregister_alg(&rtk_cbc_alg);
}


int rtk_aes_probe(struct device* pdev)
{
	int ret;
	mcp_device=pdev;

	ret = crypto_register_alg(&rtk_ecb_alg);
	if (ret)
		return ret;


	ret = crypto_register_alg(&rtk_cbc_alg);
	if (ret)
	{
		crypto_unregister_alg(&rtk_ecb_alg);
		return ret;
	}



	printk(KERN_NOTICE "rtk aes crypto engine enabled\n");
	return 0;

}

