#ifndef _MHLTX_H_
#define _MHLTX_H_


#define __RTK_MHL_GENERIC_DEBUG__  0

#if __RTK_MHL_GENERIC_DEBUG__
#define MHL_DEBUG(format, ...) printk("[MHLTx_DBG]" format "\n", ## __VA_ARGS__)
#else
#define MHL_DEBUG(format, ...) 
#endif

#define MHL_ERROR(format, ...) printk(KERN_ERR "[MHLTx_ERR]" format "\n", ## __VA_ARGS__)
#define MHL_INFO(format, ...) printk(KERN_WARNING "[MHLTx]" format "\n", ## __VA_ARGS__)

#define WR_REG_32(base, offset, val)    writel(val,(volatile unsigned int*)(base + offset))
#define RD_REG_32(base, offset)         readl((volatile unsigned int*)(base + offset))

#define FLD_GET(val, start, end) (((val) & FLD_MASK(start, end)) >> (end))
#define FLD_MASK(start, end)    (((1 << (start - end + 1)) - 1) << (end))
#define SETBITS(base,offset, Mask) WR_REG_32(base, offset, ( RD_REG_32(base,offset) | Mask))
#define READBITS(value, Mask) ((value >> (Mask)) & 0x1)
#define CLEARBITS(base,offset, Mask) WR_REG_32(base, offset,((RD_REG_32(base,offset) & ~(Mask))))
#define MASK_REG_32(base,offset, andMask, orMask) WR_REG_32(base,offset, ((RD_REG_32(base,offset) & (andMask)) | (orMask)))

#define MSC_TX_WORK 0x1
#define MSC_RX_WORK 0x2	

#define CBUS_MSC_CHANNEL 0x0
#define CBUS_DDC_CHANNEL 0x1

enum
{
     HPD_PULL_OUT	 	 = 0x00
    ,HPD_PLUG_IN   	     = 0x01    
    // this one must be last
    ,HPD_MAX
};

enum
{
     NON_BLOCKED 	 = 0x00
    ,BLOCKED   	     = 0x01    
};

/* MHL ioctl */
enum {
	MHL_GET_SINK_CAPABILITY,
	MHL_GET_RAW_EDID,
	MHL_GET_LINK_STATUS,
	MHL_GET_VIDEO_CONFIG,
	MHL_SEND_AVMUTE,
	MHL_CONFIG_TV_SYSTEM,
	MHL_CONFIG_AVI_INFO,
	MHL_SET_FREQUNCY,
	MHL_SEND_AUDIO_MUTE,
	MHL_SEND_AUDIO_VSDB_DATA,
	MHL_SEND_AUDIO_EDID2,
	MHL_CHECK_TMDS_SRC,
	MHL_SEND_MSC_CMD,
	MHL_RECEIVE_MSC_MSG,
};

#define SRC0 0x0
#define SRC1 0x1
#define SRC2 0x2
#define SRC3 0x3
#define SRC4 0x4
#define SRC5 0x5
#define SRC6 0x6
#define SRC7 0x7

#define DDC_READ_DATA_LEN 8

//DDC command type
#define DDC_READ			 	0x0
#define DDC_SEGMENT_READ		0x1
#define DDC_SHORT_READ			0x2
#define DDC_CURRENT_READ		0x3
#define DDC_WRITE				0x4

#define COMMAND			0x0
#define DATA			0x1
#define COMMAND_OFFSET	0x2
#define STOP_EOF		0x3

//IRQ
#define FW0_FIN_IRQ_EN  CBUS_TX_INT_fw0_fin_irq_en_shift
#define DDC_FIN_IRQ_EN  CBUS_TX_INT_ddc_fin_irq_en_shift

//DDC_ERR_CODE
#define ERROR_IN_CMD_OR_IMCOMPLETE_PACKET 0x4
#define PROTOCOL_ERROR	0x2 
#define RETRY_THRESHOLD_EXCEEDED 0x1

#define SCPU_INT 0x1
#define ACPU_INT 0x2

struct ddc_msg{
	int segval;
	int offset;
	int devaddr;
	unsigned char *buf;
};

//struct msc_msg{
//	unsigned char cmd2 ;
//	unsigned char offset;
//	unsigned char cmd1 ;
//	unsigned char buf[2];
//	int buf_len;
//};

struct msc_msg{
	unsigned int cmd2 ;
	unsigned int offset;
	unsigned int cmd1 ;
	unsigned char buf[2];
	unsigned int buf_len;
};

#define XFER_MODE_NONE 		0x0
#define XFER_MODE_IDLE 		0x1
#define XFER_MODE_RECV 		0x2 
#define XFER_MODE_READY 	0x3
#define XFER_MODE_ERROR 	0x4
#define XFER_MODE_TIMEOUT 	0x5
#define XFER_MODE_OK 	    0x6

struct ddc_xfer{

	unsigned char*      tx_buff;
    unsigned short      tx_buff_len;
    unsigned short      tx_len;        
    unsigned char*      rx_buff;
    unsigned short      rx_buff_len;
    unsigned short      rx_len;    
	unsigned int      	completed;	
	unsigned int 		curr_msg_num;
	unsigned int 		mode;
};


typedef struct 
{
unsigned int     reserved_0:8;
unsigned int     fw0_cmd2:8;
unsigned int     fw0_offset:8;
unsigned int     fw0_cmd1:8;
}FW0_CMD_T;

typedef struct 
{
unsigned int     fw0_req:1;
unsigned int     fw0_tx_case:3;
unsigned int     fw0_wait_case:2;
unsigned int     fw0_head:2;
unsigned int     fw0_fifo_clr:1;
unsigned int     reserved_0:23;
}FW0_CMD_INF_T;

typedef struct 
{
unsigned int     fw0_cmd_irq_en;
unsigned int     fw0_data_irq_en;
}FW0_IRQ_T;

typedef struct msc_tx_xfer{

	unsigned int mode;
	FW0_CMD_INF_T info;
	FW0_CMD_T cmd;
	//FW0_STATUS status;
	FW0_IRQ_T irq_en;
	unsigned char buf[2];
	unsigned int buf_len;
	
	unsigned char res_cmd;
	unsigned char res_data[2];
	unsigned int res_data_len;
	unsigned int err;
	unsigned int status;
}msc_tx_xfer_t;

typedef struct _CBusQueue_t
{
    uint8_t head;   // queue empty condition head == tail
    uint8_t tail;
    struct msc_tx_xfer queue[5];
}CBusQueue_t;


struct msc_msc_msg
{	
	unsigned char rcp_tmp_key_id;
	unsigned char rcp_key_id;
};


struct device_capability
{
	//DEVCAP_OFFSET_DEV_STATE 		= 0x00
	unsigned char DEV_STATE;
    
	//DEVCAP_OFFSET_MHL_VERSION		= 0x01
	unsigned char MHL_VERSION_MAJOR;
	unsigned char MHL_VERSION_MINOR;
	
    //DEVCAP_OFFSET_DEV_CAT          = 0x02
	unsigned char  DEV_TYPE;	//sink,source,dongle,
	bool POW;
	unsigned char MIN_VBUS_POWER;  //PLIM0,PLIM1
	
    //DEVCAP_OFFSET_ADOPTER_ID_H     = 0x03
	unsigned char ADOPTER_ID_H;
	
    //DEVCAP_OFFSET_ADOPTER_ID_L     = 0x04
	unsigned char ADOPTER_ID_L;
	
    //DEVCAP_OFFSET_VID_LINK_MODE    = 0x05
	bool SUPP_RGB444;
	bool SUPP_YCBCR444;
	bool SUPP_YCBCR422;
	bool SUPP_PackedPixel;
	bool SUPP_ISLANDS;
	bool SUPP_VGA;
	
    //DEVCAP_OFFSET_AUD_LINK_MODE    = 0x06
	bool AUD_2CH;
	bool AUD_8CH;
	
    //DEVCAP_OFFSET_VIDEO_TYPE       = 0x07
	bool VT_GRAPHICS;
	bool VT_PHOTO;
	bool VT_CINEMA;
	bool VT_GAME;
	bool SUPP_VT;	
		
    //DEVCAP_OFFSET_LOG_DEV_MAP      = 0x08
	bool LD_DISPLAY;
	bool LD_VIDEO;
	bool LD_AUDIO;
	bool LD_MEDIA;
	bool LD_TUNER;
	bool LD_RECORD;
	bool LD_SPEAKER;
	bool LD_GUI;
	
    //DEVCAP_OFFSET_BANDWIDTH        = 0x09
	unsigned char BANDWIDTH;
	
    //DEVCAP_OFFSET_FEATURE_FLAG     = 0x0A
	bool RCP_SUPPORT;
	bool RAP_SUPPORT;
	bool SP_SUPPORT;
	bool UCP_SEND_SUPPORT;
	bool UCP_RECV_SUPPORT;
	
    //DEVCAP_OFFSET_DEVICE_ID_H      = 0x0B
	unsigned char DEVICE_ID_H;
	
    //DEVCAP_OFFSET_DEVICE_ID_L      = 0x0C
	unsigned char DEVICE_ID_L;
	
    //DEVCAP_OFFSET_SCRATCHPAD_SIZE  = 0x0D
	unsigned char SCRATCHPAD_SIZE;

    //DEVCAP_OFFSET_INT_STAT_SIZE    = 0x0E
	unsigned char INT_SIZE;
	unsigned char STAT_SIZE;
	
    //DEVCAP_OFFSET_RESERVED         = 0x0F
};



/*******************************************
** structure to hold operating information of MhlTx component
*******************************************/
struct mhlTx_config
{
	unsigned char aucDevCapCache[16];
	struct device_capability  devcap;
	
	unsigned char rcp_tmp_key_id;
	unsigned char rcp_key_id;

};


struct cbus_adapter{
	
	void __iomem *reg_base;
	int timeout;			/* in jiffies */
	int retries;
	int ddc_msg_xfer_num;
	int ddc_msg_queue_num;
	struct ddc_xfer xfer;
	struct msc_tx_xfer msc_xfer;
	struct mhlTx_config rtk_mhltx_config;
};

struct mhltx_delayed_work {
        struct delayed_work work;
        int event;
		__u64 rx_event;
};

extern struct cbus_adapter *rtk_adap;
extern struct msc_msc_msg msc_msg;

#endif /* _MHLTX_H_ */
