#include <linux/module.h>
#include <linux/uaccess.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/slab.h>


#include <linux/jiffies.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <rbus/reg_sb2.h>

#include "mhltx.h"
#include "mhltx_dev.h"
#include "mhltx_def.h"
#include "cbus_tx_reg.h"
#include "cbus_wrapper_reg.h"
#include "iso_reg.h"
#include <hdmitx_api.h>
#include <hdmitx.h>

#define	MHL_DEV_LD_DISPLAY					(0x01 << 0)
#define	MHL_DEV_LD_VIDEO					(0x01 << 1)
#define	MHL_DEV_LD_AUDIO					(0x01 << 2)
#define	MHL_DEV_LD_MEDIA					(0x01 << 3)
#define	MHL_DEV_LD_TUNER					(0x01 << 4)
#define	MHL_DEV_LD_RECORD					(0x01 << 5)
#define	MHL_DEV_LD_SPEAKER					(0x01 << 6)
#define	MHL_DEV_LD_GUI						(0x01 << 7)

#define	MHL_MAX_RCP_KEY_CODE		(0x7F + 1)	
#define	MHL_LOGICAL_DEVICE_MAP		(MHL_DEV_LD_VIDEO | MHL_DEV_LD_GUI )

unsigned char rcpSupportTable [MHL_MAX_RCP_KEY_CODE] = {
	(MHL_DEV_LD_GUI),		// 0x00 = Select
	(MHL_DEV_LD_GUI),		// 0x01 = Up
	(MHL_DEV_LD_GUI),		// 0x02 = Down
	(MHL_DEV_LD_GUI),		// 0x03 = Left
	(MHL_DEV_LD_GUI),		// 0x04 = Right
	0, 0, 0, 0,				// 05-08 Reserved
	(MHL_DEV_LD_GUI),		// 0x09 = Root Menu
	0, 0, 0,				// 0A-0C Reserved
	(MHL_DEV_LD_GUI),		// 0x0D = Select
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	// 0E-1F Reserved
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),	// Numeric keys 0x20-0x29
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),
	0,						// 0x2A = Dot
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),	// Enter key = 0x2B
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA | MHL_DEV_LD_TUNER),	// Clear key = 0x2C
	0, 0, 0,				// 2D-2F Reserved
	(MHL_DEV_LD_TUNER),		// 0x30 = Channel Up
	(MHL_DEV_LD_TUNER),		// 0x31 = Channel Dn
	(MHL_DEV_LD_TUNER),		// 0x32 = Previous Channel
	(MHL_DEV_LD_AUDIO),		// 0x33 = Sound Select
	0,						// 0x34 = Input Select
	0,						// 0x35 = Show Information
	0,						// 0x36 = Help
	0,						// 0x37 = Page Up
	0,						// 0x38 = Page Down
	0, 0, 0, 0, 0, 0, 0,	// 0x39-0x3F Reserved
	0,						// 0x40 = Undefined

	(MHL_DEV_LD_SPEAKER),	// 0x41 = Volume Up
	(MHL_DEV_LD_SPEAKER),	// 0x42 = Volume Down
	(MHL_DEV_LD_SPEAKER),	// 0x43 = Mute
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO),	// 0x44 = Play
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_RECORD),	// 0x45 = Stop
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_RECORD),	// 0x46 = Pause
	(MHL_DEV_LD_RECORD),	// 0x47 = Record
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO),	// 0x48 = Rewind
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO),	// 0x49 = Fast Forward
	(MHL_DEV_LD_MEDIA),		// 0x4A = Eject
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA),	// 0x4B = Forward
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_MEDIA),	// 0x4C = Backward
	0, 0, 0,				// 4D-4F Reserved
	0,						// 0x50 = Angle
	0,						// 0x51 = Subpicture
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 52-5F Reserved
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO),	// 0x60 = Play Function
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO),	// 0x61 = Pause the Play Function
	(MHL_DEV_LD_RECORD),	// 0x62 = Record Function
	(MHL_DEV_LD_RECORD),	// 0x63 = Pause the Record Function
	(MHL_DEV_LD_VIDEO | MHL_DEV_LD_AUDIO | MHL_DEV_LD_RECORD),	// 0x64 = Stop Function

	(MHL_DEV_LD_SPEAKER),	// 0x65 = Mute Function
	(MHL_DEV_LD_SPEAKER),	// 0x66 = Restore Mute Function
	0, 0, 0, 0, 0, 0, 0, 0, 0, 	                        // 0x67-0x6F Undefined or reserved
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 		// 0x70-0x7F Undefined or reserved
};

bool PutNextCBusTransaction(struct msc_tx_xfer *pXfer);
struct msc_tx_xfer *GetNextCBusTransaction(void);
void CBUS_MSC_prepare_packet(struct msc_msg *pReq);
bool CBUS_Send_MSC_Command(struct cbus_adapter *adap,struct msc_msg *pReq);
bool CBusIsBusy(int channel_type);
void CBUS_MSC_disable_msc_isr(void);
void CBUS_MSC_init_Rx_work(void);
void CBUS_device_capability_handler(unsigned char data);
void enable_cbus(void);

struct cbus_adapter *rtk_adap;
CBusQueue_t CBusQueue;
struct msc_msc_msg msc_msg;


int cbus_force_rcp(void)
{
	return Force_RPC_TOAGENT_HDMI_Config_TV_System(27,HDMI_MODE_MHL);	
}

#if 1
void cbus_hw_init(void)
{
	
	WR_REG_32( 0xfe000000 ,0x24 , 0x01819100);   //DISP_PLL_DIV2_reg
	WR_REG_32( 0xfe000000 ,0x190 , 0x0000001);   //PLL_HDMI_reg
	WR_REG_32( 0xfe000000 ,0x190 , 0x0000001b);
	
}

void cbus_discovery_detect_1K(void)
{ 			    
	unsigned int _800omhs_mask=0,_800omhs_unmask=0;
	unsigned int _1200omhs_mask=0,_1200omhs_unmask=0;
	unsigned int ret;
	
	MHL_DEBUG("%s\n",__FUNCTION__);
			
	SETBITS(rtk_adap ->reg_base, CBUS_CTRL10, CBUS_CTRL10_fw_r1k_det_en_mask);
	MHL_DEBUG("set 0x180370dc[1]=0x%x \n",RD_REG_32(rtk_adap ->reg_base, CBUS_CTRL10));
		
	SETBITS(rtk_adap ->reg_base, CBUS_TX_INT, CBUS_TX_INT_det_z1k_irq_en_mask);

	//check 0.8 connect		
	if(is_revinfo_0 || is_revinfo_1)
	{
		_800omhs_mask   = ~(PHY_CTRL_reg_mhl_r10k_test_tx_mask | //~bit14/28
							PHY_CTRL_reg_r1k_vth_sel_tx_mask); 
							
		_800omhs_unmask = ( PHY_CTRL_reg_en_bias_txrx_mask | //bit 17/29/31
							PHY_CTRL_reg_mhl_r1k_det_tx_mask | 
							PHY_CTRL_reg_mhl_bmos_en_tx_mask);	
		
		MHL_DEBUG("set 0x180370D0[31,29,~28,17,~14]=");	
		
	}		
	else
	{
		_800omhs_mask   = ~( PHY_CTRL_reg_r1k_vth_sel_tx_mask); //~bit 28	
														
		_800omhs_unmask = ( PHY_CTRL_reg_mhl_r10k_test_tx_mask | //bit 14/17/29/31
							PHY_CTRL_reg_en_bias_txrx_mask |	
							PHY_CTRL_reg_mhl_r1k_det_tx_mask | 
							PHY_CTRL_reg_mhl_bmos_en_tx_mask);	
							
		MHL_DEBUG("set 0x180370D0[31,29,~28,17,14]=");					
	}
	
	MASK_REG_32(rtk_adap ->reg_base, PHY_CTRL, _800omhs_mask, _800omhs_unmask);						 							  
	MHL_DEBUG("0x%x \n",RD_REG_32(rtk_adap ->reg_base,PHY_CTRL));	

	ret= READBITS(RD_REG_32(rtk_adap ->reg_base,PHY_STATUS),PHY_STATUS_rego_mhl_r1k_det_tx_shift);
	MHL_DEBUG("PHY_STATUS=0x%x; 0.8 connect %s\n",RD_REG_32(rtk_adap ->reg_base,PHY_STATUS),ret?"ok":"failed");

	if (is_revinfo_0 || is_revinfo_1)
	{
		_1200omhs_mask   = ~(PHY_CTRL_reg_mhl_r10k_test_tx_mask); //~bit 14 
							
		_1200omhs_unmask = ( PHY_CTRL_reg_en_bias_txrx_mask | //bit 17/28/29/31
							 PHY_CTRL_reg_r1k_vth_sel_tx_mask |	
							 PHY_CTRL_reg_mhl_r1k_det_tx_mask | 
							 PHY_CTRL_reg_mhl_bmos_en_tx_mask);	
		
		MHL_DEBUG("set 0x180370D0[31,29,28,17,~14]=");	
		
	}
	else
	{
		_1200omhs_mask   = 0xffffffff; 
															
		_1200omhs_unmask = ( PHY_CTRL_reg_mhl_r10k_test_tx_mask | //bit 14/17/28/29/31
							PHY_CTRL_reg_r1k_vth_sel_tx_mask |
							PHY_CTRL_reg_en_bias_txrx_mask |	
							PHY_CTRL_reg_mhl_r1k_det_tx_mask | 
							PHY_CTRL_reg_mhl_bmos_en_tx_mask);	
							
		MHL_DEBUG("set 0x180370D0[31,29,28,17,14]=");			
	}
	
	MASK_REG_32(rtk_adap ->reg_base, PHY_CTRL,	_1200omhs_mask,_1200omhs_unmask);						
    MHL_DEBUG("0x%x \n",RD_REG_32(rtk_adap ->reg_base,PHY_CTRL));	
	
	ret= READBITS(RD_REG_32(rtk_adap ->reg_base,PHY_STATUS),PHY_STATUS_rego_mhl_r1k_det_tx_shift);
	MHL_DEBUG("PHY_STATUS=0x%x; 1.2 connect %s\n",RD_REG_32(rtk_adap ->reg_base,PHY_STATUS),ret?"ok":"failed");	
   		
}

void cbus_discovery_wakeup(void)
{
	printk("%s\n",__FUNCTION__);

	SETBITS(rtk_adap ->reg_base, CBUS_TX_INT, CBUS_TX_INT_path_en_irq_en_mask);	
	
	MASK_REG_32(rtk_adap -> reg_base ,PHY_CTRL, ~( PHY_CTRL_reg_r1k_vth_sel_tx_mask |    //bit 28/29
	                                               PHY_CTRL_reg_mhl_r1k_det_tx_mask ) 
						                      ,  ( PHY_CTRL_reg_mhl_r10k_test_tx_mask|  //bit 14/17/31
											       PHY_CTRL_reg_en_bias_txrx_mask| 
												   PHY_CTRL_reg_mhl_bmos_en_tx_mask)); 
																								   
    WR_REG_32( rtk_adap ->reg_base ,CBUS_CTRL10, 0x3);
	
}

bool cbus_check_discovery_status(void)
{
	return (FLD_GET(RD_REG_32(rtk_adap ->reg_base,CBUS_STATUS),3,1) == SRC6)  ? true :false;	
}

void cbus_HPD_handler(int HPD_event, int det_1k)
{
	
	void __iomem * base;
	base = ioremap(ISR_reg, 0x4);
	
	MHL_DEBUG("[%s] cable %s \n", __FUNCTION__,(HPD_event==1)?"pluged in":"pulled out");
	
	if( HPD_event == HPD_PULL_OUT)
	{	
				
		mhltx_switchdev_set_HPD_status(HPD_PULL_OUT);
		mhltx_switchdev_HPD_handler();	

		CBUS_MSC_disable_msc_isr();	
		WR_REG_32( rtk_adap -> reg_base ,CBUS_TX_INT1 ,0xffffffff);
		WR_REG_32( rtk_adap -> reg_base ,CBUS_TX_INT  ,0x0);		
						
		hdmitx_turn_off_tmds(HDMI_MODE_MHL);
			
		//reset cbus: to SRC0.
		CLEARBITS( base, SOFT_RESET,SOFT_RESET_rstn_cbustx_mask);
		SETBITS( base, SOFT_RESET,SOFT_RESET_rstn_cbustx_mask);
		if(det_1k)
			cbus_discovery_detect_1K();
	}	
	else
	{				
		if(cbus_check_discovery_status())
		{	
			MHL_INFO("SRC6 \n");
			WR_REG_32( rtk_adap -> reg_base ,CBUS_TX_INT1 ,0xffffffff);
			WR_REG_32( rtk_adap -> reg_base ,CBUS_TX_INT  ,0x0);
			//WR_REG_32( rtk_adap -> reg_base ,MSC_IRQ ,0xffffffff);
			
			enable_cbus();
			CBUS_MSC_init_Rx_work();
				
			mhltx_switchdev_set_HPD_status(HPD_PLUG_IN);						
			mhltx_switchdev_HPD_handler();
		}
	}

}
#endif

void enable_cbus(void)
{
//    Wreg32(0x1800708C, (1<<6)|(1<<5)|(1<<4));// clk_en_cbus_osc | clk_en_cbus_sys | clk_en_cbustx_sys
//    Wreg32(0x18007088, (1<<13)|(1<<6)|(1<<5)); // rstn_cbus | rstn_cbusrx | rstn_cbustx
    WR_REG_32( rtk_adap ->reg_base ,CBUS_CTRL9, 996); // clk_1k_div(996)
    mdelay(100);
}


void cbus_dump_ddc_msg(struct ddc_msg* msgs)
{	
	printk("msgs->segval  = 0x%x \n\
			msgs->devaddr = 0x%x \n\
			msgs->offset  = 0x%x \n", msgs->segval, msgs->devaddr, msgs->offset);		
}

void cbus_dump_DDC_STATUS(void)
{
	int ddc_status = RD_REG_32(rtk_adap ->reg_base,DDC_STATUS);
	
	printk("DDC_STATUS_get_ddc_wp =%x \n",DDC_STATUS_get_ddc_wp(ddc_status));
	printk("DDC_STATUS_get_ddc_rp =%x \n",DDC_STATUS_get_ddc_rp(ddc_status));
	printk("DDC_STATUS_get_ddc_fifo_full =%x \n",DDC_STATUS_get_ddc_fifo_full(ddc_status));
	printk("DDC_STATUS_get_ddc_fifo_empty=%x \n",DDC_STATUS_get_ddc_fifo_empty(ddc_status));
}

void cbus_set_DDC_CMD_INF(void)
{
	
	WR_REG_32( rtk_adap ->reg_base ,DDC_CMD_INF, DDC_CMD_INF_ddc_rdy(1)|  //1:ready
										DDC_CMD_INF_ddc_cmd_type(DDC_READ)|
										DDC_CMD_INF_ddc_num(DDC_READ_DATA_LEN -1));
	
	//MHL_DEBUG("cbus_set_DDC_CMD_INF =0x%x\n", RD_REG_32(rtk_adap ->reg_base,DDC_CMD_INF));
}

void cbus_set_DDC_CMD_DATA(struct ddc_msg* msgs)
{	
	//cbus_dump_ddc_msg(msgs);
		
	WR_REG_32( rtk_adap ->reg_base, DDC_CMD_DATA, DDC_CMD_DATA_ddc_segval(msgs->segval)|
										 DDC_CMD_DATA_ddc_offset(msgs->offset)|	
										 DDC_CMD_DATA_ddc_devaddr(msgs->devaddr));
}

                                             
void cbus_set_CBUS_TX_INT(int irq ,int enable)
{	
	int cbus_tx_int = RD_REG_32(rtk_adap ->reg_base, CBUS_TX_INT);
	
	switch (irq) {
	case FW0_FIN_IRQ_EN:
		cbus_tx_int |= CBUS_TX_INT_fw0_fin_irq_en(enable);	
		break;
	case DDC_FIN_IRQ_EN:
		if(enable)
			cbus_tx_int |= CBUS_TX_INT_ddc_fin_irq_en(enable);
		else
		{		
			CLEARBITS(rtk_adap ->reg_base,CBUS_TX_INT,CBUS_TX_INT_ddc_fin_irq_en_mask);					
			return;
		}	
		break;		
	default:
		break;
	}
	
	WR_REG_32( rtk_adap ->reg_base, CBUS_TX_INT ,cbus_tx_int);
	
}

void cbus_set_CONTROL(int type, int enable)
{
	int val = RD_REG_32(rtk_adap ->reg_base, CONTROL);
	
	switch (type) {
	case SCPU_INT:
		val |= CONTROL_int_scpu_en(enable);	
		break;
	case ACPU_INT:
		val |= CONTROL_int_acpu_en(enable);
		break;	
	default:
		break;
	}
	
	WR_REG_32( rtk_adap ->reg_base, CONTROL ,val);	
}

unsigned char cbus_get_DDC_RD(void)
{
	return DDC_RD_get_ddc_rdata(RD_REG_32(rtk_adap ->reg_base,DDC_RD));
}

struct cbus_adapter *cbus_get_adapter(void)
{	
	if (!rtk_adap)
		return NULL;		
		
	return rtk_adap;
}

void cbus_reset_xfer_info(struct cbus_adapter *p_this)
{
	
	p_this-> xfer.tx_buff = NULL;
    p_this-> xfer.tx_buff_len = 0;
    p_this-> xfer.tx_len = 0;        
    p_this-> xfer.rx_buff = NULL;
    p_this-> xfer.rx_buff_len = 0;
    p_this-> xfer.rx_len = 0;    
	p_this-> xfer.completed = 0;	
	p_this-> xfer.curr_msg_num = 0;
	p_this-> xfer.mode= XFER_MODE_READY;
}


void cbus_enable_scpu_int(void)
{
	cbus_set_CONTROL(SCPU_INT, 1);  
}

void cbus_enable_acpu_int(void)
{
	cbus_set_CONTROL(ACPU_INT, 1);
}

int cbus_ddc_start_xfer(struct cbus_adapter* p_this, struct ddc_msg* msgs, int num)
{
	unsigned long orig_jiffies;
		
	/* Retry automatically on arbitration loss */
	orig_jiffies = jiffies;
	
	while( p_this -> xfer.curr_msg_num < num ){
		
		//MHL_DEBUG("Before: %2d xfer of %2d (mode=%d): 0x18007080=%x,0x1800707C=%x,\n",p_this ->xfer.curr_msg_num,num,p_this ->xfer.mode,RD_REG_32(0xfe037000,0x80),RD_REG_32(0xfe037000,0x7C));
		if(p_this-> xfer.mode == XFER_MODE_READY)
		{	
            mdelay(10);	 		
			p_this-> xfer.tx_buff = msgs[p_this ->xfer.curr_msg_num].buf;		
			cbus_set_DDC_CMD_DATA(&msgs[p_this ->xfer.curr_msg_num]);		
			cbus_set_DDC_CMD_INF();	
			p_this-> xfer.mode = XFER_MODE_RECV;
			
			MHL_DEBUG("After : %2d xfer of %2d (mode=%d): 0x18007080=%x,0x1800707C=%x,\n",p_this ->xfer.curr_msg_num,num,p_this ->xfer.mode,RD_REG_32(0xfe037000,0x80),RD_REG_32(0xfe037000,0x7C));
				
		}
		if(p_this-> xfer.mode == XFER_MODE_ERROR)
		{			
			MHL_ERROR("After : %2d xfer of %2d (mode=%d): 0x18007080=%x,0x1800707C=%x,\n",p_this ->xfer.curr_msg_num,num,p_this ->xfer.mode,RD_REG_32(0xfe037000,0x80),RD_REG_32(0xfe037000,0x7C));
			return -EFAULT;
		}	
		if ( p_this-> xfer.mode == XFER_MODE_RECV && time_after(jiffies, orig_jiffies + p_this->timeout))
		{	
			MHL_ERROR("jiffies=%x orig_jiffies=%x adap->timeout=%x\n",jiffies, orig_jiffies, p_this->timeout);
			return -ETIME;
		}
	}
	
	return p_this ->xfer.curr_msg_num;
}


int __cbus_transfer(struct cbus_adapter *adap, struct ddc_msg *msgs, int num)
{	
	int ret, try;

	//enable_cbus();
	//cbus_enable_scpu_int();	
	cbus_set_CBUS_TX_INT(DDC_FIN_IRQ_EN, 1);
	cbus_reset_xfer_info(adap);
	CBUS_MSC_disable_msc_isr();


	for (ret = 0, try = 0; try <= adap->retries; try++) {
		ret = cbus_ddc_start_xfer(adap, msgs, num);
		
		if (ret == -ETIME)
		{
			MHL_ERROR("TIME OUT\n");
			break;
		}
		else if (ret == -EFAULT)
		{
			MHL_ERROR("DDC ERROR\n");
			break;
		}
	}
    cbus_set_CBUS_TX_INT(DDC_FIN_IRQ_EN, 0);
	CBUS_MSC_init_Rx_work();
	return ret;

}

void cbus_ddc_get_data(struct cbus_adapter* p_this)
{
	int ddc_status = RD_REG_32(rtk_adap ->reg_base, DDC_STATUS);	
	int i=0;
  
	//cbus_dump_DDC_STATUS();
	
	if(DDC_STATUS_get_ddc_err_code(ddc_status) == ERROR_IN_CMD_OR_IMCOMPLETE_PACKET ||
	   DDC_STATUS_get_ddc_err_code(ddc_status) == PROTOCOL_ERROR ||
	   DDC_STATUS_get_ddc_err_code(ddc_status) == RETRY_THRESHOLD_EXCEEDED)
	{		
			MHL_ERROR("ddc error status=%x \n",ddc_status);
			p_this-> xfer.mode = XFER_MODE_ERROR;
			return;
	}

    if((DDC_STATUS_get_ddc_wp(ddc_status) == DDC_READ_DATA_LEN) &&   // check ptr
       (DDC_STATUS_get_ddc_rp(ddc_status) == 0) &&
	   (DDC_STATUS_get_ddc_fifo_full(ddc_status) == 0) &&
       (DDC_STATUS_get_ddc_fifo_empty(ddc_status) == 0)) 
    {            
            for(i=0;i<DDC_READ_DATA_LEN;i++)
            {					
				p_this-> xfer.tx_buff[i] = cbus_get_DDC_RD();
				p_this-> xfer.tx_len ++;
				//MHL_DEBUG("p_this->xfer->tx_buff[%d]=0x%x \n",i,p_this-> xfer.tx_buff[i]);					
            }
			
			p_this -> xfer.curr_msg_num ++;
			p_this -> xfer.mode = XFER_MODE_READY;
			MHL_DEBUG("p_this ->xfer.curr_msg_num=%d\n",p_this ->xfer.curr_msg_num);		
			MHL_DEBUG("p_this-> xfer.tx_len=%d\n",p_this-> xfer.tx_len);
			MHL_DEBUG("p_this-> xfer.mode=%d\n",p_this-> xfer.mode);			
    } 
    
	if((DDC_STATUS_get_ddc_wp(ddc_status) == DDC_READ_DATA_LEN) &&
       (DDC_STATUS_get_ddc_rp(ddc_status) == DDC_READ_DATA_LEN) &&
       (DDC_STATUS_get_ddc_fifo_full(ddc_status) == 0) &&
       (DDC_STATUS_get_ddc_fifo_empty(ddc_status) == DDC_STATUS_ddc_fifo_full_mask))  
	{
			p_this-> xfer.completed = 1;
			MHL_DEBUG("read complete \n");
    }  
	
}

/*************************************
**
** MSC Channel of CBUS
**
*************************************/
void CBUS_send_MSC_GET_MSC_ERRORCODE(void)
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);

	req.cmd1    = MHL_GET_MSC_ERRORCODE;
	
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSC_SET_INT( unsigned char interrupt_registers , unsigned char mask)
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);

	req.cmd2  	= 0;
	req.cmd1    = MHL_SET_INT;
	req.offset  = interrupt_registers;
	req.buf[0]  = mask;
	req.buf_len  = 1;
	
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSC_WRITE_STAT( unsigned char status_registers, unsigned char value)
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);
	
    req.cmd2  	= 0;
	req.cmd1    = MHL_WRITE_STAT;
	req.offset  = status_registers;
	req.buf[0]  = value;
	req.buf_len  = 1;
 	
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSC_READ_DEVCAP( unsigned char capability_registers)
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);
	
	req.cmd2  	= 0;
	req.cmd1    = MHL_READ_DEVCAP;
	req.offset  = capability_registers;
	req.buf[0]  = 0;
	req.buf_len  = 0;
	
	rtk_adap-> msc_xfer.mode = BLOCKED;
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSC_WRITE_BURST( unsigned char Scratchapd_registers , unsigned char value)
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);
	
	req.cmd2  	= 0;
	req.cmd1    = MHL_WRITE_BURST;
	req.offset  = Scratchapd_registers;
	req.buf[0]  = value;
	req.buf_len  = 1;
	
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSG_MSC_MSG (unsigned char *cmdData )
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);

	req.cmd2  	= 0;
	req.cmd1    = MHL_MSC_MSG;
	req.offset  = 0;
	req.buf[0]  = cmdData[0];
	req.buf[1]  = cmdData[1];
	req.buf_len  = 2;
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSC_SET_HPD (void )
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);

	req.cmd1    = MHL_SET_HPD;	
	CBUS_Send_MSC_Command(rtk_adap,&req);
}

void CBUS_send_MSC_ACK (void)
{	
	int val =  RD_REG_32( rtk_adap ->reg_base, MSC_CTRL);		
	struct msc_msg 	req={0};
	
	if(MSC_CTRL_get_mcumsc_en(val))
	{
		//MHL_DEBUG("%s \n", __FUNCTION__);
		req.cmd1    = MHL_ACK;	
		CBUS_Send_MSC_Command(rtk_adap,&req);	
	}
}

void CBUS_send_MSC_ABORT (void)
{
	struct msc_msg 	req={0};
	
	//MHL_DEBUG("%s \n", __FUNCTION__);

	req.cmd1    = MHL_ABORT;	
	CBUS_Send_MSC_Command(rtk_adap,&req);	
}

void CBUS_reply_MSC_SET_INT(unsigned char offset,unsigned char value)
{
	
	int devcap_write = RD_REG_32( rtk_adap ->reg_base, DEVCAP_WRITE);
	int cap_offset = RD_REG_32( rtk_adap ->reg_base,DEVCAP_OFFSET);
	
	if(offset<= DEVCAP_OFFSET_get_int_offset(cap_offset))
	{	
		switch (offset)
		{		
			case MHL_RCHANGE_INT:			
			case MHL_DCHANGE_INT:
			
				WR_REG_32( rtk_adap ->reg_base, MSC_DEVCAP8, devcap_write |
															 DEVCAP_WRITE_msc_offset_adr(offset)|
															 DEVCAP_WRITE_devcap_write(1)|
															 DEVCAP_WRITE_msc_reg_data(value));	
				break;
							
			default:
				MHL_ERROR("Unknown MSC_SET_INT cmd\n");
				break;
		}
		
		CBUS_send_MSC_ACK();   
	}
	else
		CBUS_send_MSC_ABORT();
		

}

void CBUS_reply_MSC_WRITE_STAT(unsigned char offset,unsigned char value)
{

	//int devcap_write = RD_REG_32( rtk_adap ->reg_base, DEVCAP_WRITE);	
	int cap_offset = RD_REG_32( rtk_adap ->reg_base,DEVCAP_OFFSET);
			
	//MHL_DEBUG("%s \n", __FUNCTION__);	
	//MHL_DEBUG("0xfe037120=%x \n",*(volatile unsigned int*)0xfe037120);	
	//MHL_DEBUG("0xfe037098=%x \n",*(volatile unsigned int*)0xfe037098);
	
	if(offset<= DEVCAP_OFFSET_sta_offset(cap_offset))
	{
		switch (offset)
		{		
			case MHL_STATUS_REG_CONNECTED_RDY :					
			case MHL_STATUS_REG_LINK_MODE:
				
				/*WR_REG_32( rtk_adap ->reg_base, DEVCAP_WRITE, devcap_write |
															  DEVCAP_WRITE_msc_offset_adr(offset)|
															  DEVCAP_WRITE_devcap_write(1)|
															  DEVCAP_WRITE_msc_reg_data(value));	*/
				//MHL_DEBUG("0xfe037120=%x \n",*(volatile unsigned int*)0xfe037120);	
				//MHL_DEBUG("0xfe037098=%x \n",*(volatile unsigned int*)0xfe037098);										 
				break;
							
			default:
				MHL_ERROR("Unknown MSC_WRITE_STAT cmd\n");
				break;
		}
	
		CBUS_send_MSC_ACK();
	}
	else
		CBUS_send_MSC_ABORT();
	
}


void CBUS_reply_MSC_READ_DEVCAP(unsigned char offset)
{
	struct msc_msg msg={0};
	unsigned char data=0;
	int cap_offset = RD_REG_32( rtk_adap ->reg_base,DEVCAP_OFFSET);
	int devcap0 = RD_REG_32( rtk_adap ->reg_base,MSC_DEVCAP0);
	int devcap1 = RD_REG_32( rtk_adap ->reg_base,MSC_DEVCAP1);
	int devcap2 = RD_REG_32( rtk_adap ->reg_base,MSC_DEVCAP2);
	int devcap3 = RD_REG_32( rtk_adap ->reg_base,MSC_DEVCAP3);

	if(offset <= DEVCAP_OFFSET_get_cap_offset(cap_offset))
	{
	
		switch ( offset )
		{		
			case DEVCAP_OFFSET_DEV_STATE :				
				data = MSC_DEVCAP0_get_dev_state(devcap0);
				break;
				
			case DEVCAP_OFFSET_MHL_VERSION :
				data = MSC_DEVCAP0_get_mhl_ver(devcap0);
				break;
				
			case DEVCAP_OFFSET_DEV_CAT :
				data = (MSC_DEVCAP0_get_dev_type(devcap0)|
					   (MSC_DEVCAP0_get_pow(devcap0)<<4)|
					   (MSC_DEVCAP0_get_plim(devcap0)<<5));		
				break;
				
			case DEVCAP_OFFSET_ADOPTER_ID_H :
				data = MSC_DEVCAP0_get_adop_id_h(devcap0);
				break;
				
			case DEVCAP_OFFSET_ADOPTER_ID_L :
				data = 	MSC_DEVCAP1_get_adop_id_l(devcap1);
				break;
		
			case DEVCAP_OFFSET_VID_LINK_MODE :
				data = (MSC_DEVCAP1_get_supp_rgb444(devcap1)|
					   (MSC_DEVCAP1_get_supp_yuv444(devcap1)<<1)|
					   (MSC_DEVCAP1_get_supp_yuv422(devcap1)<<2)|
					   (MSC_DEVCAP1_get_supp_ppixel(devcap1)<<3)|
					   (MSC_DEVCAP1_get_supp_islands(devcap1)<<4)|
					   (MSC_DEVCAP1_get_supp_vga(devcap1)<<5));			
				break;
				
			case DEVCAP_OFFSET_AUD_LINK_MODE :
				data = (MSC_DEVCAP1_get_aud_2ch(devcap1)|
					   (MSC_DEVCAP1_get_aud_8ch(devcap1)<<1));
				break;
				
			case DEVCAP_OFFSET_VIDEO_TYPE :
				
				break;
				
			case DEVCAP_OFFSET_LOG_DEV_MAP :
				data = (MSC_DEVCAP2_get_ld_display(devcap2)|
					   (MSC_DEVCAP2_get_ld_video(devcap2)<<1)|
					   (MSC_DEVCAP2_get_ld_audio(devcap2)<<2)|
					   (MSC_DEVCAP2_get_ld_media(devcap2)<<3)|
					   (MSC_DEVCAP2_get_ld_tuner(devcap2)<<4)|
					   (MSC_DEVCAP2_get_ld_record(devcap2)<<5)|
					   (MSC_DEVCAP2_get_ld_speaker(devcap2)<<6)|
					   (MSC_DEVCAP2_get_ld_gui(devcap2)<<7));
				break;
				
			case DEVCAP_OFFSET_BANDWIDTH :
				break;
				
			case DEVCAP_OFFSET_FEATURE_FLAG:
				data = (MSC_DEVCAP2_get_rcp_supp(devcap2)|
					   (MSC_DEVCAP2_get_rap_supp(devcap2)<<1)|
					   (MSC_DEVCAP2_get_sp_supp(devcap2)<<2)|
					   (MSC_DEVCAP2_get_ucp_send_supp(devcap2)<<3)|
					   (MSC_DEVCAP2_get_ucp_recv_supp(devcap2)<<4));
				break;
				
			case DEVCAP_OFFSET_DEVICE_ID_H :
				data = MSC_DEVCAP2_get_device_id_h(devcap2);
				break;
				
			case DEVCAP_OFFSET_DEVICE_ID_L :
				data = MSC_DEVCAP3_get_device_id_l(devcap3);
				break;
			case DEVCAP_OFFSET_SCRATCHPAD_SIZE:
				data = MSC_DEVCAP3_get_scratchpad_size(devcap3);
				break;
				
			case DEVCAP_OFFSET_INT_STAT_SIZE:
				data = (MSC_DEVCAP3_get_int_size(devcap3)|
					   (MSC_DEVCAP3_get_stat_size(devcap3)>>4));
				break;
				
			case DEVCAP_OFFSET_RESERVED :
				break;   
				
			default:
				MHL_ERROR("Unknown MSC_READ_DEVCAP cmd\n");
				break;
		}
		
		MHL_DEBUG("%s: offset=%x data=%x\n",__FUNCTION__,offset,data);
		msg.cmd1 = MHL_ACK ;
		msg.buf[0] = data;
		msg.buf_len = 1;
	}
	else
		CBUS_send_MSC_ABORT();
			
}

void CBUS_reply_MSC_SetClr_HPD(void)
{
	CBUS_send_MSC_ACK();   				
}

void CBUS_reply_MSC_GET_STATE(void)
{
	struct msc_msg msg={0};	
	int devcap0 = RD_REG_32( rtk_adap ->reg_base,MSC_DEVCAP0);
	unsigned char data= MSC_DEVCAP0_get_dev_state(devcap0);
		
	msg.buf[0] = data;
	msg.buf_len = 1;
	CBUS_Send_MSC_Command(rtk_adap,&msg);
}


void CBUS_reply_MSC_GET_VENDOR_ID(void)
{
	struct msc_msg msg={0};		
	int cbus_ctrl2 = RD_REG_32( rtk_adap ->reg_base,CBUS_CTRL2);
	unsigned char data = CBUS_CTRL2_get_vendor_id(cbus_ctrl2);
		
	msg.buf[0] = data;
	msg.buf_len = 1;
	CBUS_Send_MSC_Command(rtk_adap,&msg);	
}

void CBUS_reply_MSC_GET_DDC_ERRORCODE(void)
{    
	struct msc_msg msg={0};
	int err = RD_REG_32( rtk_adap ->reg_base,DDC_STATUS);	 //0x1803708c; not 0x180370A0
	unsigned char data = DDC_STATUS_get_ddc_err_code(err);
	
	msg.buf[0] = data;
	msg.buf_len = 1;
	CBUS_Send_MSC_Command(rtk_adap,&msg);

}

void CBUS_reply_MSC_GET_MSC_ERRORCODE(void)
{
	struct msc_msg msg={0};
	int err = RD_REG_32( rtk_adap ->reg_base,MSC_BURST_DATA);	//0x1803706C; not 0x180370A4
	unsigned char data = MSC_BURST_DATA_get_msc_err_code(err);
	
	msg.buf[0] = data;
	msg.buf_len = 1;
	CBUS_Send_MSC_Command(rtk_adap,&msg);
	
}
void CBUS_update_MSG_MSC_MSG_INFO(void)
{
	if(MHL_LOGICAL_DEVICE_MAP & rcpSupportTable [msc_msg.rcp_tmp_key_id & 0x7F] )
		msc_msg.rcp_key_id = msc_msg.rcp_tmp_key_id;
}

void CBUS_reply_MSG_MSC_MSG_RCP(void)
{
	struct msc_msg msg={0};
	int status =RD_REG_32( rtk_adap ->reg_base, MSG_STATUS2);
	unsigned char key_id =MSG_STATUS2_get_rcp_rcv(status);
	
	if(1)
	{	
		msg.cmd1 = MHL_ACK;	
		CBUS_MSC_prepare_packet(&msg);
						
		memset(&msg,0x0,sizeof(msg));
		msg.cmd1 = MHL_MSC_MSG;
		msg.offset = MHL_MSC_MSG_RCPK;	
		msg.buf[0] = key_id;			
		msg.buf_len = 1;		
		msc_msg.rcp_tmp_key_id = key_id;	
		CBUS_Send_MSC_Command(rtk_adap,&msg);
				
	}
	else
	{
		msg.buf[0] = MHL_MSC_MSG_RCPE;
		msg.buf_len = 1;		
		
		msg.buf[0] = MHL_MSC_MSG_RCPE_UNSUP_ACT_CODE;		
	

		msg.buf[0] = MHL_MSC_MSG_RCPK;		

		
		msg.buf[0] = key_id;	
	
	}	

}

void CBUS_reply_MSG_MSC_MSG(int sub_cmd)
{
	
	switch (sub_cmd)
	{		
		case MHL_MSC_MSG_MSGE :	
			break;
		
		case MHL_MSC_MSG_RCP:
			CBUS_reply_MSG_MSC_MSG_RCP();				
			break;
			
		case MHL_MSC_MSG_RCPK:
		case MHL_MSC_MSG_RCPE:
			break;
		
		case MHL_MSC_MSG_RAP:
		case MHL_MSC_MSG_RAPK:
			break;
			
		case MHL_MSC_MSG_UCP:
		case MHL_MSC_MSG_UCPK:
		case MHL_MSC_MSG_UCPE:
			break;	
						
		default:
			MHL_ERROR("Unknown MSC_MSG_MSGE sub_cmd\n");
			break;
	}
}

void CBUS_MSC_init_Rx_work(void)
{
	//set MSC capability register enable 0x18037068
	int val =  RD_REG_32( rtk_adap ->reg_base, MSC_CTRL);
	int msc_irq_en =  RD_REG_32( rtk_adap ->reg_base, MSC_IRQ_EN);
	
	WR_REG_32( rtk_adap ->reg_base, MSC_CTRL, val|MSC_CTRL_mcumsc_en(1) );
	//printk("0xfe037068=%x \n",*(volatile unsigned int*)0xfe037068);
	
	//clear native msc rx fifo? status-bit
	
#if 1	
    WR_REG_32( rtk_adap -> reg_base ,MSC_IRQ ,0xffffffff);	
	//enable INT 0x1803705C
	WR_REG_32( rtk_adap ->reg_base, MSC_IRQ_EN , msc_irq_en| 
												 MSC_IRQ_EN_wr_stat_irq_en(1)|
												 MSC_IRQ_EN_rd_devcap_irq_en(0)|
												 MSC_IRQ_EN_get_stat_irq_en(0)|
												 MSC_IRQ_EN_get_id_irq_en(0)|
												 MSC_IRQ_EN_get_msg_irq_en(0)|
												 MSC_IRQ_EN_sc1_err_irq_en(0)|
												 MSC_IRQ_EN_ddc_err_irq_en(0)|
												 MSC_IRQ_EN_msc_err_irq_en(0)|
												 MSC_IRQ_EN_wr_burst_irq_en(0)|
												 MSC_IRQ_EN_sc3_err_irq_en(0)|
												 //
												 MSC_IRQ_EN_fwdef_cmd_irq_en(0)|
												 MSC_IRQ_EN_allmsc_cmd_irq_en(0)|
												 //
												 MSC_IRQ_EN_abort_res_irq_en(0)|
												 MSC_IRQ_EN_abort_req_irq_en(0)|
												 MSC_IRQ_EN_ddc_abort_irq_en(0)|
												 MSC_IRQ_EN_ddc_nack_irq_en(0)|
												 //
												 MSC_IRQ_EN_msge_irq_en(0)|
												 //rcp,rcpk,rcpe,
												 MSC_IRQ_EN_rcp_irq_en(1)|
												 MSC_IRQ_EN_rcpk_irq_en(0)|
												 MSC_IRQ_EN_rcpe_irq_en(0)|
												 //rap
												 MSC_IRQ_EN_rap_irq_en(0)|
												 MSC_IRQ_EN_rapk_irq_en(0)|
												 //ucp,ucpe,ucpk,
												 MSC_IRQ_EN_ucp_irq_en(0)|
												 MSC_IRQ_EN_ucpe_irq_en(0)|
												 MSC_IRQ_EN_ucpk_irq_en(0)|
												 //
												 MSC_IRQ_EN_msg_sub_irq_en(0)|
												 //
												 MSC_IRQ_EN_det_rxsense_irq_en(0)|
												 MSC_IRQ_EN_msc_hpd_irq_en(1));
#endif
												
}

void CBUS_MSC_disable_msc_isr(void)
{
	//disable INT 0x1803705C
	WR_REG_32( rtk_adap ->reg_base, MSC_IRQ_EN , 0);
											
}

void CBUS_MSC_receive_packet_responder(__u64 rx_event)
{
	int msc_rev_data, msc_rev_cmd;
	unsigned char offset, len, cmd, data[2];		
					 
	if(READBITS(rx_event, MSC_IRQ_wr_stat))
	{
		msc_rev_cmd = RD_REG_32( rtk_adap ->reg_base, MSC_RCV_CMD);	
		cmd 		= MSC_RCV_CMD_get_na_msc_cmd(msc_rev_cmd);	
		offset      = MSC_RCV_CMD_get_na_msc_offset(msc_rev_cmd);
		len    		= MSC_RCV_CMD_get_na_rx_len(msc_rev_cmd);
		
		msc_rev_data = RD_REG_32( rtk_adap ->reg_base, MSC_RCV_DATA);
		data[0]   	 = MSC_RCV_DATA_get_rx_fifo_data(msc_rev_data);
				
		MHL_INFO("Responder receive MSC_WRITE_STAT: cmd=%x offset=%x data=%x len=%x\n ",cmd,offset,data[0],len);
   		
		if(offset >= MHL_STATUS_REG_CONNECTED_RDY)
			CBUS_reply_MSC_WRITE_STAT(offset, data[0]);			
		else
			CBUS_reply_MSC_SET_INT(offset, data[0]);           
    }
	
	if(READBITS(rx_event, MSC_IRQ_msc_hpd))
	{	
		msc_rev_cmd = RD_REG_32( rtk_adap ->reg_base, MSC_RCV_CMD);	
		cmd 		= MSC_RCV_CMD_get_na_msc_cmd(msc_rev_cmd);
		
		MHL_INFO("Responder receive MSC_HPD cmd=%x\n",cmd);
		// 0x64	- Set Hot Plug Detect in follower
		// 0x65	- Clear Hot Plug Detect in follower
		if(cmd == MHL_CLR_HPD)
		{	
			CBUS_reply_MSC_SetClr_HPD();		
			mhltx_switchdev_set_HPD_status(HPD_PULL_OUT);
			mhltx_switchdev_HPD_handler();
		}
		else
		{
			CBUS_reply_MSC_SetClr_HPD();		
			mhltx_switchdev_set_HPD_status(HPD_PLUG_IN);
			mhltx_switchdev_HPD_handler();
		}
	}
	
	if(READBITS(rx_event, MSC_IRQ_rd_devcap))
	{
		msc_rev_cmd = RD_REG_32( rtk_adap ->reg_base, MSC_RCV_CMD);
		offset = MSC_RCV_CMD_get_na_msc_offset(msc_rev_cmd);
		MHL_INFO("Responder receive MSC_READ_DEVCAP: offset=%x \n ",offset);
		
		CBUS_reply_MSC_READ_DEVCAP(offset);		              		
    }

	
 	if(READBITS(rx_event, MSC_IRQ_get_stat)) // 0x62 -
	{
		MHL_INFO("Responder receive MSC_GET_STATE\n ");
	//	CBUS_reply_MSC_GET_STATE();			              	
    }	

	if(READBITS(rx_event, MSC_IRQ_get_id)) // 0x63 - for vendor id
	{
		MHL_INFO("Responder receive MSC_GET_VENDOR_ID\n ");
		//CBUS_reply_MSC_GET_VENDOR_ID();			              	
    }	
		
	if(READBITS(rx_event, MSC_IRQ_ddc_err)) // 0x6A	- Get DDC channel command error code.
	{
		MHL_INFO("Responder receive MSC_GET_DDC_ERRORCODE\n ");
		//CBUS_reply_MSC_GET_DDC_ERRORCODE();			              	
    }		
	
	if(READBITS(rx_event, MSC_IRQ_msc_err)) // 0x6B	- Get MSC command error code.
	{
		MHL_INFO("Responder receive MSC_GET_MSC_ERRORCODE\n ");
		//CBUS_reply_MSC_GET_MSC_ERRORCODE();			              	
    }		
		
	if(READBITS(rx_event, MSC_IRQ_rcp)) // 0x6B	- Get MSC command error code.
	{
		msc_rev_cmd = RD_REG_32( rtk_adap ->reg_base, MSC_RCV_CMD);	
		cmd 		= MSC_RCV_CMD_get_na_msc_cmd(msc_rev_cmd);
		MHL_INFO("Responder receive MSG_MSC_MSG_RCP cmd=%x\n",cmd);					
		
		CBUS_reply_MSG_MSC_MSG(MHL_MSC_MSG_RCP);			              	
    }

}

/***********************************************
**
**   MSC Tx work
**
***********************************************/

void CBUS_MSC_receive_packet_requester(void)
{
	int fw0_status  = RD_REG_32( rtk_adap ->reg_base, FW0_STATUS);
		
	switch (rtk_adap -> msc_xfer.cmd.fw0_cmd1)
    {
		//case MHL_SET_INT:	// Set one interrupt register = 0x60
		case MHL_WRITE_STAT:	// Write one status register = 0x60 | 0x80	
		case MHL_SET_HPD:			// 0x64	- Set Hot Plug Detect in follower
		case MHL_CLR_HPD:			// 0x65	- Clear Hot Plug Detect in follower
			rtk_adap -> msc_xfer.res_cmd = FW0_STATUS_get_fw0_cmd_rcv(fw0_status);//Ack or Abort
			MHL_INFO("Requester(%x) receive(cmd)=(%2x)\n ",rtk_adap -> msc_xfer.cmd.fw0_cmd1
														   ,rtk_adap -> msc_xfer.res_cmd);
            break;
						    
	    case MHL_READ_DEVCAP:	// Read one device capability register = 0x61
           rtk_adap -> msc_xfer.res_cmd = FW0_STATUS_get_fw0_cmd_rcv(fw0_status);//Ack or Abort
		   rtk_adap -> msc_xfer.res_data[0] = FW0_STATUS_get_fw0_data_rcv(fw0_status); //one byte Data
		   MHL_INFO("Requester(%x) receive(cmd,data)=(%2x,%2x)\n ",rtk_adap -> msc_xfer.cmd.fw0_cmd1
																   ,rtk_adap -> msc_xfer.res_cmd 
																   ,rtk_adap -> msc_xfer.res_data[0]);	
           CBUS_device_capability_handler(rtk_adap -> msc_xfer.res_data[0]);
			break;
 					
		case MHL_GET_STATE:			// 0x62 -				
		case MHL_GET_VENDOR_ID:		// 0x63 - for vendor id								
		case MHL_GET_DDC_ERRORCODE:		// 0x6A	- Get DDC channel command error code.								
		case MHL_GET_MSC_ERRORCODE:		// 0x6B	- Get MSC command error code.
			rtk_adap -> msc_xfer.res_data[0] = FW0_STATUS_get_fw0_data_rcv(fw0_status); //one byte Data
			MHL_INFO("Requester(%x) receive(data)=(%2x)\n " ,rtk_adap -> msc_xfer.cmd.fw0_cmd1
															 ,rtk_adap -> msc_xfer.res_data[0]);																	
            break;

        case MHL_MSC_MSG:
			rtk_adap -> msc_xfer.res_cmd = FW0_STATUS_get_fw0_cmd_rcv(fw0_status);//Ack or Abort
			MHL_INFO("Requester(%x) receive(cmd)=(%2x)\n ",rtk_adap -> msc_xfer.cmd.fw0_cmd1
						  							      ,rtk_adap -> msc_xfer.res_cmd);	
			if(rtk_adap -> msc_xfer.res_cmd==MHL_ACK)	
				CBUS_update_MSG_MSC_MSG_INFO();
			break;
			
        case MHL_WRITE_BURST:
           
            break;
			
		case MHL_ACK:					
		case MHL_NACK:						
		case MHL_ABORT:
			MHL_INFO("Requester send cmd=%x\n",rtk_adap -> msc_xfer.cmd.fw0_cmd1);
			break;

        default:
			MHL_INFO("Requester send unknown cmd=%x\n",rtk_adap -> msc_xfer.cmd.fw0_cmd1);
            break;
    }
			
}

void CBUS_MSC_dump_packet(struct msc_msg *msg)
{
	
	printk("dump msc_msg: cmd1 =%2x offset=%2x buf=%2x%2x buf_len=%d cmd2 =%2x \n", msg->cmd1, 
																				   msg->offset,						
																				   msg->buf[0], msg->buf[0],
																				   msg->buf_len,
                                                                                   msg->cmd2);
}

void CBUS_MSC_dump_tx_xfer(struct msc_tx_xfer *pXfer)
{
	printk("pXfer ->cmd.fw0_cmd2=%x \n",pXfer -> cmd.fw0_cmd2);
	printk("pXfer ->cmd.fw0_offset=%x\n",pXfer -> cmd.fw0_offset);
	printk("pXfer ->cmd.fw0_cmd1=%x \n",pXfer -> cmd.fw0_cmd1);
	
	printk("pXfer ->buf[0]=%x \n",pXfer -> buf[0]);
	printk("pXfer ->buf[1]=%x \n",pXfer ->buf[1]);
	printk("pXfer ->buf_len=%x \n",pXfer -> buf_len);
		
	printk("pXfer -> info.fw0_tx_case=%x\n",pXfer -> info.fw0_tx_case);
	printk("pXfer -> info.fw0_wait_case=%x\n",pXfer -> info.fw0_wait_case);

	printk("pXfer -> irq_en.fw0_cmd_irq_en=%x\n",pXfer -> irq_en.fw0_cmd_irq_en);
	printk("pXfer -> irq_en.fw0_data_irq_en =%x\n",pXfer -> irq_en.fw0_data_irq_en);
}

void CBUS_MSC_tx_work(void)
{
	struct msc_tx_xfer *pxfer;		
	unsigned int fw0_tx_case,fw0_wait_case,cmd_irq_en,data_irq_en; 		
	unsigned int cbus_tx_int  = RD_REG_32( rtk_adap ->reg_base, CBUS_TX_INT);	 
	int count=0;
	
	if(rtk_adap-> msc_xfer.status == XFER_MODE_IDLE)
	{	
		while(QUEUE_DEPTH(CBusQueue) >0 )
		{
			if(count>0)			
				mdelay(10);
							
			pxfer = GetNextCBusTransaction();
						
			//memcpy(&rtk_adap ->msc_xfer,pxfer,sizeof(struct msc_tx_xfer));			
			memcpy(&rtk_adap ->msc_xfer.cmd, &pxfer->cmd,sizeof(pxfer->cmd));			
			
			fw0_tx_case   = pxfer->info.fw0_tx_case;			
			fw0_wait_case = pxfer->info.fw0_wait_case;
			cmd_irq_en    = pxfer->irq_en.fw0_cmd_irq_en;
			data_irq_en   = pxfer->irq_en.fw0_data_irq_en;
						
			
			//clear fifo:0x1803_7028
			WR_REG_32( rtk_adap ->reg_base, FW0_CMD_INF, FW0_CMD_INF_fw0_fifo_clr(1));	
						
			//fill packet: 0x1803_702C
			WR_REG_32( rtk_adap ->reg_base, FW0_CMD, FW0_CMD_fw0_cmd2(pxfer->cmd.fw0_cmd2)|
													FW0_CMD_fw0_offset(pxfer->cmd.fw0_offset)|	
													FW0_CMD_fw0_cmd1(pxfer->cmd.fw0_cmd1));		
			//printk("0xfe03702c=%x \n",*(volatile unsigned int*)0xfe03702c);		
		
			while(pxfer->buf_len !=0)
			{	
				//0x1803_7034										 
				WR_REG_32( rtk_adap ->reg_base, FW0_WDATA , FW0_WDATA_fw0_data(pxfer->buf[ 1-(pxfer->buf_len)]));
				pxfer->buf_len --;					
				
			}
	
			//check	MSC_CTRL_rx_fifo_clr?
			//printk("0xfe037068=%x \n",*(volatile unsigned int*)0xfe037068);	
			//clear read-only status flag			
			//printk("0xfe037058=%x \n",*(volatile unsigned int*)0xfe037058);
			
			WR_REG_32( rtk_adap ->reg_base, CBUS_TX_INT1, CBUS_TX_INT1_fw0_fin_irq(1) |
														CBUS_TX_INT1_fw0_cmd_irq(1) |
														CBUS_TX_INT1_fw0_data_irq(1) |
														CBUS_TX_INT1_ddc_fin_irq(1) |
														CBUS_TX_INT1_clk_mode_chg(1)|
														CBUS_TX_INT1_dcap_rdy_chg(1)|
														CBUS_TX_INT1_muted_chg(1)|
														CBUS_TX_INT1_path_en_chg (1) |
														CBUS_TX_INT1_det_z1k_irq(1)|
														CBUS_TX_INT1_sink_det_irq(1) |                                                                            
														CBUS_TX_INT1_parity_err_irq(1) |                                                                         
														CBUS_TX_INT1_disconn_irq(1) |                                                                                   
														CBUS_TX_INT1_det_hpd_irq(1) );
			//printk("0xfe037094=%x \n",*(volatile unsigned int*)0xfe037094);											  
			
			
			//enable interrupt: 0x18037090            	
 			setbits( rtk_adap ->reg_base, CBUS_TX_INT, CBUS_TX_INT_fw0_fin_irq_en (1) |
													   CBUS_TX_INT_fw0_cmd_irq_en (cmd_irq_en) |
													   CBUS_TX_INT_fw0_data_irq_en(data_irq_en) );			          
			//printk("0xfe037090=%x \n",*(volatile unsigned int*)0xfe037090);												
		
		
			//start FW requester : 0x18037028	
			WR_REG_32( rtk_adap ->reg_base, FW0_CMD_INF, (FW0_CMD_INF_fw0_req (FW0_REQ_ENABLED) |
														FW0_CMD_INF_fw0_tx_case (fw0_tx_case) |
														FW0_CMD_INF_fw0_wait_case(fw0_wait_case) | 
														FW0_CMD_INF_fw0_head(FW0_HEADER_MSC_PACKET)) );													
			//printk("0xfe037028=%x \n",*(volatile unsigned int*)0xfe037028);
			count++;
		}				
			
			//rtk_adap-> msc_xfer.status = XFER_MODE_READY;

	}

	else if (rtk_adap-> msc_xfer.status == XFER_MODE_READY)
	{	
		
		if(CBUS_TX_INT_get_fw0_fin_irq_en(cbus_tx_int)==1)
		{
			CBUS_MSC_receive_packet_requester();
			rtk_adap -> msc_xfer.status = XFER_MODE_OK;
				
			CLEARBITS(rtk_adap ->reg_base, CBUS_TX_INT , (CBUS_TX_INT_fw0_data_irq_en_mask|
														  CBUS_TX_INT_fw0_fin_irq_en_mask|
														  CBUS_TX_INT_fw0_cmd_irq_en_mask));
		}
		else
		{
			rtk_adap -> msc_xfer.status = XFER_MODE_IDLE;
			MHL_INFO("skip tx work\n");		
		}
					
	}
	else
		MHL_ERROR("MSC channel status=%x\n",rtk_adap-> msc_xfer.status);

}

void CBUS_MSC_rx_work(__u64 rx_event)
{
	CBUS_MSC_receive_packet_responder(rx_event);
}

void CBUS_MSC_prepare_packet(struct msc_msg *pReq)
{ 
	//MHL_DEBUG("%s  %d \n",__FUNCTION__,__LINE__);
	
	struct msc_tx_xfer Xfer={0};

	Xfer.cmd.fw0_cmd2 = pReq->cmd2;
	Xfer.cmd.fw0_offset = pReq->offset;
	Xfer.cmd.fw0_cmd1 = pReq->cmd1 ;
	Xfer.buf[0] = pReq->buf[0];
	Xfer.buf[1] = pReq->buf[1];
	Xfer.buf_len = pReq->buf_len;
	    
	Xfer.irq_en.fw0_cmd_irq_en = FW0_CMD_IRQ_ENABLED ;
	Xfer.irq_en.fw0_data_irq_en = FW0_DATA_IRQ_DISABLED ;
		
	switch (Xfer.cmd.fw0_cmd1)
    {
        case MHL_WRITE_STAT:	// Write one status register = 0x60
//or	case MHL_SET_INT:	// Set one interrupt register = 0x60
            Xfer.info.fw0_tx_case = FW0_TX_CASE_CMD1_OFF_DATA;
			Xfer.info.fw0_wait_case = FW0_WAIT_CASE_CMD;
            break;

        case MHL_READ_DEVCAP:	// Read one device capability register = 0x61
            Xfer.info.fw0_tx_case = FW0_TX_CASE_CMD1_OFF;
			Xfer.info.fw0_wait_case = FW0_WAIT_CASE_CMD_DATA;
			Xfer.irq_en.fw0_cmd_irq_en = FW0_CMD_IRQ_DISABLED ;
			Xfer.irq_en.fw0_data_irq_en = FW0_DATA_IRQ_ENABLED;
            break;
 		
		case MHL_SET_HPD:			// 0x64	- Set Hot Plug Detect in follower
		case MHL_CLR_HPD:			// 0x65	- Clear Hot Plug Detect in follower
			Xfer .info.fw0_tx_case = FW0_TX_CASE_CMD1;
			Xfer .info.fw0_wait_case = FW0_WAIT_CASE_CMD;
            break;
		
		case MHL_GET_STATE:			// 0x62 -
		case MHL_GET_VENDOR_ID:		// 0x63 - for vendor id
		case MHL_GET_DDC_ERRORCODE:		// 0x6A	- Get DDC channel command error code.
		case MHL_GET_MSC_ERRORCODE:		// 0x6B	- Get MSC command error code.
			Xfer.info.fw0_tx_case = FW0_TX_CASE_CMD1;
			Xfer.info.fw0_wait_case = FW0_WAIT_CASE_DATA;
			Xfer.irq_en.fw0_cmd_irq_en = FW0_CMD_IRQ_DISABLED ;
			Xfer.irq_en.fw0_data_irq_en = FW0_DATA_IRQ_ENABLED;	
            break;

        case MHL_MSC_MSG:
			Xfer.info.fw0_tx_case = FW0_TX_CASE_CMD1_OFF_DATA;
            Xfer.info.fw0_wait_case = FW0_WAIT_CASE_CMD;
            break;

        case MHL_WRITE_BURST:
            Xfer.info.fw0_tx_case = FW0_TX_CASE_CMD1_OFF_DATA_CMD2;
            Xfer.info.fw0_wait_case = FW0_WAIT_CASE_CMD;
            break;
		
        case MHL_ACK:
		case MHL_NACK:
		case MHL_ABORT:
			Xfer.info.fw0_tx_case = FW0_TX_CASE_CMD1;
            Xfer.info.fw0_wait_case = FW0_WAIT_CASE_NO;
			Xfer.irq_en.fw0_cmd_irq_en = FW0_CMD_IRQ_DISABLED;
			break;
		
        default:
			MHL_ERROR("Unknown cmd=%x\n", Xfer.cmd.fw0_cmd1);
           break;
    }

//	CBUS_MSC_dump_tx_xfer(&Xfer);
	PutNextCBusTransaction(&Xfer); 
}

bool PutNextCBusTransaction(struct msc_tx_xfer *pXfer)
{
	//struct msc_tx_xfer *pXfer1;
	//int i;
    if (QUEUE_FULL(CBusQueue))
    {
        //queue is full
        return false;
    }
    // at least one slot available
    CBusQueue.queue[CBusQueue.tail] = *pXfer;
    ADVANCE_QUEUE_TAIL(CBusQueue)
		
	/*for(i=0; i<= CBusQueue.tail; i++)
	{
		pXfer1=&CBusQueue.queue[i];
		CBUS_MSC_dump_tx_xfer(pXfer1);
	}
	*/	
    return true;
}

struct msc_tx_xfer *GetNextCBusTransaction(void)
{
    if (0==QUEUE_DEPTH(CBusQueue))
    {
        return NULL;
    }
    else
    {
		struct msc_tx_xfer *retVal;		
        retVal = &CBusQueue.queue[CBusQueue.head];			
        ADVANCE_QUEUE_HEAD(CBusQueue)		
        return retVal;
    }
}

void mhltx_work_queue(struct work_struct *work)
{
        struct mhltx_delayed_work *mhltx_w = container_of(work, struct mhltx_delayed_work, work.work);
        int event = mhltx_w->event;
		int rx_event = mhltx_w->rx_event;
				
		if( CBusIsBusy(CBUS_DDC_CHANNEL)== false)
		{			
			if (event == MSC_TX_WORK)
			{		
				CBUS_MSC_tx_work();				
			}
			else if (event == MSC_RX_WORK)
			{	
				CBUS_MSC_rx_work(rx_event);
			}
			else
				MHL_ERROR("[%s] unknow event = %d\n",__FUNCTION__,event);
		}
		else
			MHL_ERROR("CBus is busy\n");
	
		kfree(mhltx_w);
        mhltx_w = 0;
}

bool CBUS_Send_MSC_Command(struct cbus_adapter *adap,struct msc_msg *pReq)
{
	int ret=0;

	wait_queue_head_t   wq;
	
	init_waitqueue_head(&wq);
	
	CBUS_MSC_prepare_packet(pReq);
	
	adap-> msc_xfer.status = XFER_MODE_IDLE;
		
	CBUS_MSC_tx_work();

	if(adap-> msc_xfer.mode == BLOCKED)	
	{
		//wait_event_timeout(wq, adap-> msc_xfer.status!= XFER_MODE_IDLE, TX_TIMEOUT + 50);
		wait_event_timeout(wq, adap-> msc_xfer.status == XFER_MODE_READY, TX_TIMEOUT + 50);
	
		switch(adap->msc_xfer.status)    
			{
			case XFER_MODE_OK:
				MHL_DEBUG("%x: xmit message success\n",rtk_adap -> msc_xfer.cmd.fw0_cmd1);
				break;
									
			case XFER_MODE_ERROR:
				MHL_ERROR("%x: xmit message failed\n",rtk_adap -> msc_xfer.cmd.fw0_cmd1);
				break;
			
			case XFER_MODE_READY:
				MHL_DEBUG("%x: ready for next xmit message\n",rtk_adap -> msc_xfer.cmd.fw0_cmd1);            
				break;	
			
			default:
			case XFER_MODE_TIMEOUT:    
				MHL_ERROR("%x: xmit message timeout\n",rtk_adap -> msc_xfer.cmd.fw0_cmd1);
				break;
					
			}
		
		ret = (adap->msc_xfer.status == XFER_MODE_OK) ? 0 : -1;
   }	
   adap-> msc_xfer.status = XFER_MODE_NONE;
   adap-> msc_xfer.mode = NON_BLOCKED;
   return ret;
}
	
bool CBusIsBusy(int channel_type)
{
	int cbus_tx_int = RD_REG_32(rtk_adap ->reg_base, CBUS_TX_INT);

	if(channel_type)	
		return (CBUS_TX_INT_get_ddc_fin_irq_en(cbus_tx_int)) ? true :false;				
	else
		return (CBUS_TX_INT_get_fw0_fin_irq_en(cbus_tx_int)) ? true :false;
		
}

void CBUS_get_device_capability(void)
{

#if 0
	int i;
	
	for(i= DEVCAP_OFFSET_DEV_STATE ; i<DEVCAP_OFFSET_RESERVED ;i++)
	{
		CBUS_send_MSC_READ_DEVCAP(i);
	}	
#endif

	CBUS_send_MSC_READ_DEVCAP(DEVCAP_OFFSET_VID_LINK_MODE);	
}

void CBUS_reset_device_capability(struct device_capability *devcap)
{	
	memset(devcap,0x0,sizeof(struct device_capability));	
}

void CBUS_dump_device_capability(struct device_capability *devcap)
{
	printk("[MHLTx]dump Dev Cap Register\n");
//	printk("[MHLTx]Device State: 0x%x\n", devcap->DEV_STATE);
//	printk("[MHLTx]MHL_Version: 0x%x,0x%x\n",devcap->MHL_VERSION_MAJOR,devcap->MHL_VERSION_MINOR);
    
	
//	printk("[MHLTx]Device Category: ");
//	if(devcap->DEV_TYPE == SINK)
//		printk("sink,");
//	if(devcap->DEV_TYPE == SOURCE)
//		printk("source,");
//	if(devcap->DEV_TYPE == DONGLE)
//		printk("dongle,");	
//		
//	if(devcap->POW)	
//		printk("POW on,");
//		
//	if(devcap->MIN_VBUS_POWER == 0x0)	
//		printk("device delivers at least 500mA.");
//	if(devcap->MIN_VBUS_POWER == 0x1)	
//		printk("device delivers at least 900mA.");
//	if(devcap->MIN_VBUS_POWER == 0x2)	
//		printk("device delivers at least 1.5A.");	
//	if(devcap->MIN_VBUS_POWER == 0x3)	
//		printk("I VBUS_Dongle_to_Source{min}.");
//	printk("\n");		
//		
//		
//	printk("[MHLTx]Adopter ID: 0x%2x,0x%2x\n",devcap->ADOPTER_ID_H ,devcap->ADOPTER_ID_L);
//		
	printk("[MHLTx]Vedio Link Mode: ");
	if(devcap->SUPP_RGB444)
		printk("RGB 4:4:4,");
	if(devcap->SUPP_YCBCR444)
		printk("YCBCR 4:4:4,");
	if (devcap->SUPP_YCBCR422)
		printk("YCBCR 4:2:2,");
	if (devcap->SUPP_PackedPixel)
		printk("packed pixel,"); 
	if (devcap->SUPP_ISLANDS)
		printk("data islands,"); 
	if (devcap->SUPP_VGA)
		printk("VGA."); 	
	printk("\n");
//	
//	printk("[MHLTx]Audio Link Mode: ");	
//	if(!devcap->AUD_2CH && !devcap->AUD_8CH)
//		printk("doesn't support any audio content.\n");
//	else if(devcap->AUD_2CH && !devcap->AUD_8CH)
//		printk("supports 2 channel PCM audio streams.\n");
//	else if (devcap->AUD_2CH && devcap->AUD_8CH)
//		printk("supports greater than 2 channel PCM audio streams.\n");
//	else
//		printk("N/A\n");
//	
//
//	printk("[MHLTx]Video Type: ");	
//	if(devcap-> SUPP_VT)
//	{
//		printk("supports ");
//		if(devcap->VT_GRAPHICS)
//			printk("GRAPHICS,");
//		if(devcap->VT_PHOTO)
//			printk("PHOTO,");
//		if (devcap->VT_CINEMA)
//			printk("CINEMA,");
//		if (devcap->VT_GAME)
//			printk("GAME."); 	
//		printk("\n");	
//	}		
//	else
//		printk("N/A\n");
//		
//			
//	printk("[MHLTx]Logical Device: ");		
//	if(devcap->LD_DISPLAY)
//		printk("display output,");
//	if(devcap->LD_VIDEO)
//		printk("video output,");
//	if (devcap->LD_AUDIO)
//		printk("audio output,");
//	if (devcap->LD_MEDIA)
//		printk("media handling,");
//	if (devcap->LD_MEDIA)
//		printk("media handling,");
//	if (devcap->LD_TUNER)
//		printk("tuner handling,");		
//    if (devcap->LD_RECORD)
//		printk("content recording,");
//    if (devcap->LD_SPEAKER)
//		printk("audio output(audible sound),");
//    if (devcap->LD_GUI)
//		printk("rendering and navigation of Menus or other UI.");		
//	printk("\n");	
//		
//	printk("[MHLTx]Bandwidth: 0x%x\n",devcap->BANDWIDTH);	
//		
//	printk("[MHLTx]Feature: supports ");
//	if(devcap->RCP_SUPPORT)
//		printk("RCP,");
//	if(devcap->RAP_SUPPORT)
//		printk("RAP,");
//	if (devcap->SP_SUPPORT)
//		printk("scratchpad registers,");
//	if (devcap->UCP_SEND_SUPPORT)
//		printk("sending UCP,");
//	if (devcap->UCP_RECV_SUPPORT)
//		printk("receivinging UCP,");
//	printk("\n");
//	
//	printk("[MHLTx]Device ID: 0x%x,0x%x\n" ,devcap->DEVICE_ID_H,devcap->DEVICE_ID_L);
//	
//	if (devcap->SP_SUPPORT)
//		printk("[MHLTx]Scratchpad Size: 0x%x\n" ,devcap->SCRATCHPAD_SIZE);
//	
//	printk("[MHLTx]Interrupt Registers Count: 0x%x\n" ,devcap->INT_SIZE);
//	printk("[MHLTx]Status Registers Count: 0x%x\n" ,devcap->STAT_SIZE);
		
}



void CBUS_device_capability_handler(unsigned char data)
{	
		
	switch (rtk_adap -> msc_xfer.cmd.fw0_offset)
    {
        case DEVCAP_OFFSET_DEV_STATE:		// 0x00
			rtk_adap -> rtk_mhltx_config.devcap.DEV_STATE = data;           
            break;
			
        case DEVCAP_OFFSET_MHL_VERSION:		// 0x01            
			rtk_adap -> rtk_mhltx_config.devcap.MHL_VERSION_MAJOR = FLD_GET(data,OFFSET_MHL_VERSION_MAJOR_MSB,OFFSET_MHL_VERSION_MAJOR_LSB);
			rtk_adap -> rtk_mhltx_config.devcap.MHL_VERSION_MINOR = FLD_GET(data,OFFSET_MHL_VERSION_MINOR_MSB,OFFSET_MHL_VERSION_MINOR_LSB);
            break;
 		
		case DEVCAP_OFFSET_DEV_CAT:		//0x02		
			rtk_adap -> rtk_mhltx_config.devcap.DEV_TYPE      	= FLD_GET(data,OFFSET_DEV_TYPE_MSB,OFFSET_DEV_TYPE_LSB);
			rtk_adap -> rtk_mhltx_config.devcap.POW			  	= READBITS(data,OFFSET_POW);	
			rtk_adap -> rtk_mhltx_config.devcap.MIN_VBUS_POWER  = FLD_GET(data,OFFSET_PLIM1,OFFSET_PLIM0);
			break;
			
		case DEVCAP_OFFSET_ADOPTER_ID_H:     //0x03		
			rtk_adap -> rtk_mhltx_config.devcap.ADOPTER_ID_H = data;
            break;
		
		case DEVCAP_OFFSET_ADOPTER_ID_L:     //0x04
			rtk_adap -> rtk_mhltx_config.devcap.ADOPTER_ID_L = data;
            break;
		
		case DEVCAP_OFFSET_VID_LINK_MODE:    //0x05
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_RGB444      = READBITS(data,OFFSET_SUPP_RGB444);
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_YCBCR444    = READBITS(data,OFFSET_SUPP_YCBCR444);
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_YCBCR422    = READBITS(data,OFFSET_SUPP_YCBCR422);
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_PackedPixel = READBITS(data,OFFSET_SUPP_PackedPixel);
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_ISLANDS     = READBITS(data,OFFSET_SUPP_ISLANDS);
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_VGA         = READBITS(data,OFFSET_SUPP_VGA);			
			break;
			
		case DEVCAP_OFFSET_AUD_LINK_MODE:    //0x06
			rtk_adap -> rtk_mhltx_config.devcap.AUD_2CH = READBITS(data,OFFSET_AUD_2CH);
			rtk_adap -> rtk_mhltx_config.devcap.AUD_8CH = READBITS(data,OFFSET_AUD_8CH);
			break;

		case DEVCAP_OFFSET_VIDEO_TYPE:       //0x07
			rtk_adap -> rtk_mhltx_config.devcap.VT_PHOTO  = READBITS(data,OFFSET_VT_PHOTO);
			rtk_adap -> rtk_mhltx_config.devcap.VT_CINEMA = READBITS(data,OFFSET_VT_CINEMA);
			rtk_adap -> rtk_mhltx_config.devcap.VT_GAME   = READBITS(data,OFFSET_VT_GAME);
			rtk_adap -> rtk_mhltx_config.devcap.SUPP_VT   = READBITS(data,OFFSET_SUPP_VT);
            break;

        case DEVCAP_OFFSET_LOG_DEV_MAP:      //0x08
			rtk_adap -> rtk_mhltx_config.devcap.LD_DISPLAY = READBITS(data,OFFSET_LD_DISPLAY);
			rtk_adap -> rtk_mhltx_config.devcap.LD_VIDEO   = READBITS(data,OFFSET_LD_VIDEO);
			rtk_adap -> rtk_mhltx_config.devcap.LD_AUDIO   = READBITS(data,OFFSET_LD_AUDIO);
			rtk_adap -> rtk_mhltx_config.devcap.LD_MEDIA   = READBITS(data,OFFSET_LD_MEDIA);
			rtk_adap -> rtk_mhltx_config.devcap.LD_TUNER   = READBITS(data,OFFSET_LD_TUNER);
			rtk_adap -> rtk_mhltx_config.devcap.LD_RECORD  = READBITS(data,OFFSET_LD_RECORD);
			rtk_adap -> rtk_mhltx_config.devcap.LD_SPEAKER = READBITS(data,OFFSET_LD_SPEAKER);
			rtk_adap -> rtk_mhltx_config.devcap.LD_GUI     = READBITS(data,OFFSET_LD_GUI);
            break;

        case DEVCAP_OFFSET_BANDWIDTH:        //0x09
            rtk_adap -> rtk_mhltx_config.devcap.BANDWIDTH = data;
            break;
		
        case DEVCAP_OFFSET_FEATURE_FLAG:     //0x0A
			rtk_adap -> rtk_mhltx_config.devcap.RCP_SUPPORT = READBITS(data,OFFSET_RCP_SUPPORT);
			rtk_adap -> rtk_mhltx_config.devcap.RAP_SUPPORT = READBITS(data,OFFSET_RAP_SUPPORT);	
			rtk_adap -> rtk_mhltx_config.devcap.SP_SUPPORT  = READBITS(data,OFFSET_SP_SUPPORT);
			rtk_adap -> rtk_mhltx_config.devcap.UCP_SEND_SUPPORT = READBITS(data,OFFSET_UCP_SEND_SUPPORT);
			rtk_adap -> rtk_mhltx_config.devcap.UCP_RECV_SUPPORT = READBITS(data,OFFSET_UCP_RECV_SUPPORT);	
			break;
		
		case DEVCAP_OFFSET_DEVICE_ID_H:     //0x0B
			rtk_adap -> rtk_mhltx_config.devcap.DEVICE_ID_H = data;
			break;
			
		case DEVCAP_OFFSET_DEVICE_ID_L:     //0x0C
			rtk_adap -> rtk_mhltx_config.devcap.DEVICE_ID_L = data;
			break;
			
		case DEVCAP_OFFSET_SCRATCHPAD_SIZE:  //0x0D
			rtk_adap -> rtk_mhltx_config.devcap.SCRATCHPAD_SIZE = data;
			break;
	
		case DEVCAP_OFFSET_INT_STAT_SIZE:    //0x0E
			rtk_adap -> rtk_mhltx_config.devcap.INT_SIZE  = FLD_GET(data,OFFSET_INIT_SIZE_MSB,OFFSET_INIT_SIZE_LSB);
			rtk_adap -> rtk_mhltx_config.devcap.STAT_SIZE = FLD_GET(data,OFFSET_STAT_SIZE_MSB,OFFSET_STAT_SIZE_LSB);
			break;
			
        default:
			MHL_ERROR("%s: Unknown offset=%x\n",__FUNCTION__ ,rtk_adap -> msc_xfer.cmd.fw0_offset);
           break;
    }
		
}