#include <linux/module.h>
#include <linux/uaccess.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/delay.h>


#include <linux/miscdevice.h>
#include <linux/platform_device.h>	
#include <linux/device.h>
#include <linux/irq.h>
#include <linux/interrupt.h>

#include <linux/of.h>
#include <linux/of_device.h>
#include <linux/of_irq.h>
#include <linux/of_address.h>

#include "cbus_tx_reg.h"
#include "mhltx_dev.h"
#include "mhltx.h"
#include "mhltx_cbus.h"
#include "mhltx_def.h"

#include <hdmitx_api.h>
#include <hdmitx.h>

static int __init rtk_mhl_init(void);
static void __exit rtk_mhl_exit(void);

static mhltx_device_t tx_dev; 
static asoc_hdmi_t mhl_data;


struct delayed_work *mhltx_submit_work(int event,__u64 rx_event, int delay)
{
	struct mhltx_delayed_work *work;

        work = kmalloc(sizeof(struct mhltx_delayed_work), GFP_ATOMIC);

        if (work) {
                INIT_DELAYED_WORK(&work->work, mhltx_work_queue);
                work->event = event;
				work->rx_event = rx_event;
                queue_delayed_work(tx_dev.workqueue,
                                   &work->work,
                                   msecs_to_jiffies(delay));
        } else {
                printk(KERN_WARNING "MHLTx: Cannot allocate memory to "
                                    "create work\n");
                return 0;
        }

        return &work->work;

}
				
int ops_send_msc_cmd(void __user *arg)
{
	struct msc_msg msg;
	
	//MHL_DEBUG("[%s]MHL_SEND_MSC_CMD \n", __func__);	
	
	if (copy_from_user(& msg, arg, sizeof(msg))) {
		MHL_ERROR("[%s]send msc cmd failed to copy from user !\n", __func__);
		return -EFAULT;
	}
		
	return CBUS_Send_MSC_Command(rtk_adap,&msg);
}

int ops_receive_msc_msg(void __user *arg)
{	
	int ret= 0;
	//MHL_DEBUG("[%s]MHL_RECEIVE_MSC_MSG \n", __func__);
	
	if (copy_to_user(arg, &msc_msg.rcp_key_id ,sizeof(msc_msg.rcp_key_id))) {
		MHL_ERROR("[%s]receive msc msg failed to copy to user !\n", __func__);
		return -EFAULT;
	}
	else{
		msc_msg.rcp_key_id = 0xFF;
		return ret;	
	}
}

int mhltx_open(struct inode *inode, struct file *filp)
{	
	if (nonseekable_open(inode, filp))
		return -ENODEV;
	
	filp->private_data = &tx_dev;
	
	return 0;
}

/*------------------------------------------------------------------
 * Func : mhltx_ioctl
 *
 * Desc : ioctl function of mhltx miscdev
 *
 * Parm : file  : context of file
 *        cmd   : control command
 *        arg   : arguments
 *         
 * Retn : 0 : success, others fail  
 *------------------------------------------------------------------*/
static long mhltx_ioctl(struct file* file,unsigned int cmd, unsigned long arg)
{  
	
	mhltx_device_t* dev = (mhltx_device_t*) file->private_data;               
	asoc_hdmi_t* tx_data;
		
    if (!dev)
        return -ENODEV;
      
    tx_data = mhltx_get_drvdata(dev);

	switch (cmd) {	
	case MHL_GET_SINK_CAPABILITY:	
		return ops_get_sink_cap((void __user *)arg,tx_data);
		
	case MHL_GET_RAW_EDID:	
		return ops_get_raw_edid((void __user *)arg,tx_data);
		
	case MHL_GET_LINK_STATUS:
		return ops_get_link_status((void __user *)arg);
				
	case MHL_GET_VIDEO_CONFIG:
		return ops_get_video_config((void __user *)arg,tx_data);
	
	case MHL_SEND_AVMUTE:
		return ops_send_AVmute((void __user *)arg, dev->hdmi_base);

	case MHL_CONFIG_TV_SYSTEM:
		return ops_config_tv_system((void __user *)arg);
	
	case MHL_CONFIG_AVI_INFO:
		return ops_config_avi_info((void __user *)arg);		
	
	case MHL_SET_FREQUNCY:
		return ops_set_frequency((void __user *)arg);
		
	case MHL_SEND_AUDIO_MUTE:		
		return ops_set_audio_mute((void __user *)arg);	
	
	case MHL_SEND_AUDIO_VSDB_DATA:	
		return ops_send_audio_vsdb_data((void __user *)arg);
	
	case MHL_SEND_AUDIO_EDID2:	
		return ops_send_audio_edid2((void __user *)arg);	

	case MHL_CHECK_TMDS_SRC:	
		return ops_check_tmds_src((void __user *)arg, dev->hdmi_base);			
	
    case MHL_SEND_MSC_CMD:
		return ops_send_msc_cmd((void __user *)arg);
	
	case MHL_RECEIVE_MSC_MSG:
		return ops_receive_msc_msg((void __user *)arg);	

    default:    
        MHL_DEBUG(" unknown ioctl cmd %08x\n", cmd);        
        return -EFAULT;          
    }		
   
   return 0; 
    
}
  
/*------------------------------------------------------------------
 * mhl misc device
 *------------------------------------------------------------------*/

static struct file_operations mhltx_fops = {
        .owner 			= THIS_MODULE,
		.open	    	= mhltx_open,
        .unlocked_ioctl = mhltx_ioctl,
		.poll	    	= NULL,
};


void msc_isr_handler(void)
{	
	int msc_irq  	 = RD_REG_32( tx_dev.reg_base, MSC_IRQ);
	int msc_irq_en 	 = RD_REG_32( tx_dev.reg_base, MSC_IRQ_EN);
	__u64 rx_event=0;
	
	
	if( MSC_IRQ_get_msc_hpd_irq (msc_irq) & MSC_IRQ_EN_get_msc_hpd_irq_en(msc_irq_en))
		rx_event|= 1ULL << MSC_IRQ_msc_hpd;
	
		
	if( MSC_IRQ_get_wr_stat_irq (msc_irq) & MSC_IRQ_EN_get_wr_stat_irq_en(msc_irq_en))
		rx_event|= 1ULL << MSC_IRQ_wr_stat;
	
	
	if( MSC_IRQ_get_rd_devcap_irq (msc_irq) & MSC_IRQ_EN_get_rd_devcap_irq_en(msc_irq_en))	
		rx_event|= 1ULL << MSC_IRQ_rd_devcap;
	

	if( MSC_IRQ_get_get_stat_irq (msc_irq) & MSC_IRQ_EN_get_get_stat_irq_en(msc_irq_en))	
		rx_event|= 1ULL << MSC_IRQ_get_stat;		
	
		
	if( MSC_IRQ_get_get_id_irq (msc_irq) & MSC_IRQ_EN_get_get_id_irq_en(msc_irq_en))
		rx_event|= 1ULL << MSC_IRQ_get_id;
	
		
	if( MSC_IRQ_get_ddc_err_irq (msc_irq) & MSC_IRQ_EN_get_ddc_err_irq_en(msc_irq_en))
		rx_event|= 1ULL << MSC_IRQ_ddc_err;
	

	if( MSC_IRQ_get_rcp_irq (msc_irq) & MSC_IRQ_EN_get_rcp_irq_en(msc_irq_en))
		rx_event|= 1ULL << MSC_IRQ_rcp;
	
	//printk("0xfe037058=%x  rx_event=%llx \n",*(volatile unsigned int*)0xfe037058,rx_event);
	if(rx_event!=0)
		mhltx_submit_work(MSC_RX_WORK,rx_event,0);
				
	WR_REG_32(tx_dev.reg_base, MSC_IRQ ,msc_irq);
			
}


static irqreturn_t mhltx_isr(int irq, void *data)
{	
	int cbus_tx_int1 = RD_REG_32( tx_dev.reg_base, CBUS_TX_INT1);
    int cbus_tx_int  = RD_REG_32( tx_dev.reg_base, CBUS_TX_INT);
    int ddc_cmd_inf  = RD_REG_32( tx_dev.reg_base, DDC_CMD_INF);  

#if 0
	if( CBUS_TX_INT1_get_disconn_irq(cbus_tx_int1))	   
    {  	// have to set 0x180370dc[0] = 0 to stop discovery flow.
		cbus_HPD_handler(HPD_PULL_OUT,1);		
    }
#endif	

#if 1
	if( CBUS_TX_INT1_get_path_en_chg(cbus_tx_int1))
	{	
		CLEARBITS(tx_dev.reg_base,CBUS_TX_INT, CBUS_TX_INT_path_en_irq_en_mask);	
		cbus_HPD_handler(HPD_PLUG_IN,0);		
	}
	
	if( CBUS_TX_INT1_get_det_z1k_irq(cbus_tx_int1) && CBUS_TX_INT_get_det_z1k_irq_en(cbus_tx_int))
	{		
		CLEARBITS(tx_dev.reg_base,CBUS_TX_INT, CBUS_TX_INT_det_z1k_irq_en_mask);
		cbus_discovery_wakeup();		
	}
#endif

	if((CBUS_TX_INT1_get_fw0_data_irq(cbus_tx_int1) && CBUS_TX_INT1_get_fw0_fin_irq(cbus_tx_int1)) || 
	   (CBUS_TX_INT1_get_fw0_cmd_irq(cbus_tx_int1) && CBUS_TX_INT1_get_fw0_fin_irq(cbus_tx_int1)))
    {  	
		rtk_adap-> msc_xfer.status = XFER_MODE_READY;
		mhltx_submit_work(MSC_TX_WORK,0,0);
    }
	else if(CBUS_TX_INT1_get_fw0_fin_irq(cbus_tx_int1))
	{
		printk("skip tx work\n");	
		CLEARBITS(tx_dev.reg_base, CBUS_TX_INT, CBUS_TX_INT_fw0_fin_irq_en_mask);
	}
	
			
    if( CBUS_TX_INT1_get_ddc_fin_irq(cbus_tx_int1) && !DDC_CMD_INF_get_ddc_rdy(ddc_cmd_inf)) // ddc_fin_irq
    {
        cbus_ddc_get_data(rtk_adap);		
    }
	
	msc_isr_handler();
	
	// clear int1
    WR_REG_32( tx_dev.reg_base, CBUS_TX_INT1 ,cbus_tx_int1);
	return IRQ_HANDLED;	
}

/*------------------------------------------------------------------
 * Func : register_mhltx_miscdev
 * Desc : register mhltx miscdev
 * Parm : mhltx device : mhltx miscdev to be registered
 * Retn : 0
 *------------------------------------------------------------------*/
int register_mhltx_miscdev(mhltx_device_t* device)
{
    struct miscdevice *miscdev = &device->miscdev;
           
	miscdev->minor = MISC_DYNAMIC_MINOR;
    miscdev->name = "mhltx";
    miscdev->mode = 0666;
    miscdev->fops = &mhltx_fops;                  
    
    return misc_register(miscdev);
}

int deregister_mhltx_miscdev(mhltx_device_t* device)
{
    return misc_deregister(& device->miscdev);
}

static int rtk_mhl_probe(struct platform_device *pdev)
{	

	tx_dev.reg_base = of_iomap(pdev->dev.of_node, 0);
	if(!tx_dev.reg_base)
	{
		MHL_ERROR("Can't map MHL Tx registers\n");
		goto end;	
	}
	
	MHL_DEBUG("tx_base=%x \n",(unsigned int)tx_dev.reg_base);	
	
	
	tx_dev.hdmi_base = of_iomap(pdev->dev.of_node, 1);
	if(!tx_dev.hdmi_base)
	{
		MHL_ERROR("Can't map HDMI Tx registers\n");
		goto end;	
	}
	
	MHL_DEBUG("tx_base=%x \n",(unsigned int)tx_dev.hdmi_base);
		
	if(FLD_GET(RD_REG_32(tx_dev.reg_base,CBUS_STATUS),3,1) != 0x6)
	{     	
        MHL_ERROR("Not SRC6\n");	
        goto end;
    }
		
	rtk_adap = kmalloc(sizeof(struct cbus_adapter), GFP_KERNEL);
	if (!rtk_adap)
	{
		MHL_ERROR("malloc cbus_adapter failed\n");
		kfree(rtk_adap);
		return -EFAULT;		
	}
	else
	{
        memset(rtk_adap,0x0,sizeof(struct cbus_adapter));
		rtk_adap->reg_base = tx_dev.reg_base;
		rtk_adap->retries=3;
		rtk_adap->timeout =500;
	}

    tx_dev.workqueue = create_singlethread_workqueue("mhltx");
    if (tx_dev.workqueue == NULL)
        goto end;
	
	if (register_mhltx_miscdev(&tx_dev)) {
        MHL_ERROR("Could not register_mhltx_miscdev\n");
        goto end;
    }
	
	mhltx_set_drvdata(&tx_dev, &mhl_data);
	
	tx_dev.irq_num = irq_of_parse_and_map (pdev->dev.of_node, 0);
	if (!tx_dev.irq_num) {
		MHL_ERROR("fail to parse of irq.\n");
	}
	
	MHL_DEBUG("irq_num =%d\n",tx_dev.irq_num);
	
	if(request_irq(tx_dev.irq_num, mhltx_isr,IRQF_SHARED,"mhltx",&pdev->dev)) {
		MHL_ERROR("cannot register IRQ %d\n", tx_dev.irq_num);
		 return -ENODEV;
	}
	
	WR_REG_32( rtk_adap -> reg_base ,CBUS_TX_INT1 ,0xffffffff);
#if 0	
        //RxSense
        SETBITS(rtk_adap ->reg_base, CBUS_TX_INT, CBUS_TX_INT_disconn_irq_en_mask);
#endif
    enable_cbus();
	cbus_enable_scpu_int();	
	CBUS_MSC_init_Rx_work(); 
	
	if (register_mhltx_switchdev(&tx_dev)) {
        MHL_ERROR("Could not register_mhltx_switchdev\n");
        goto err_register;
    }
	
#ifdef USE_ION_AUDIO_HEAP
	//for RPC
	register_ion_client("mhltx_driver");
#endif	
		
	return 0;

err_register:
        deregister_mhltx_miscdev(&tx_dev);			
		
end:
    kfree(rtk_adap);
	return -EFAULT;		
	
}

static const struct of_device_id rtk_mhltx_dt_ids[] = {
     { .compatible = "Realtek,rtk119x-mhltx", },
     {},
};
MODULE_DEVICE_TABLE(of, rtk_mhltx_dt_ids);

static struct platform_driver rtk_mhl_driver = {
	.probe = rtk_mhl_probe,
	.driver = {
		.name = "rtk_mhl",
		.owner = THIS_MODULE, 
		.of_match_table = of_match_ptr(rtk_mhltx_dt_ids),
	}, 
};

/*-----------------------------------------------------------------------------
 * Function: mhl_init
 *-----------------------------------------------------------------------------
 */
static int __init rtk_mhl_init(void)
{									
    if (platform_driver_register(&rtk_mhl_driver)) {
        MHL_ERROR("Could not add mhl driver\n");
        goto err_register;
    }
						
	return 0;	
		
err_register:
        platform_driver_unregister(&rtk_mhl_driver);

    return -EFAULT;
}

/*-----------------------------------------------------------------------------
 * Function: hdcp_exit
 *-----------------------------------------------------------------------------
 */
static void __exit rtk_mhl_exit(void)
{
#ifdef USE_ION_AUDIO_HEAP
	//for RPC
	deregister_ion_client("mhltx_driver");
#endif
	kfree(rtk_adap);
	deregister_mhltx_switchdev(&tx_dev);
	deregister_mhltx_miscdev(&tx_dev);
	platform_driver_unregister(&rtk_mhl_driver);
}

/*-----------------------------------------------------------------------------
 *-----------------------------------------------------------------------------
 */
module_init(rtk_mhl_init);
module_exit(rtk_mhl_exit);
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Realtek MHL kernel module");
MODULE_AUTHOR("Wilma Wu");
