#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/switch.h>
#include <linux/device.h>

#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/slab.h>

#include <linux/of.h>
#include <linux/of_device.h>
#include <linux/of_irq.h>
#include <linux/of_gpio.h>

#include "mhltx.h"
#include "mhltx_dev.h"
#include "mhltx_cbus.h"
#include "mhltx_def.h"

#include <hdmitx.h>
#include <hdmitx_api.h>

#define MHL_SWITCH_NAME "mhl"

static struct hdmitx_switch_data s_data;
static struct switch_dev *sdev = NULL;
static unsigned int plug_in_val;
//struct work_struct tv_work;

static ssize_t mhltx_switch_print_state(struct switch_dev *sdev, char *buffer)
{
	//MHL_DEBUG("mhltx_switch_print_state \n");
	return sprintf(buffer, "%d", s_data.state);
}

//static void mhltx_switch_tv_work_func(struct work_struct *work) 
//{
//	CBUS_get_device_capability();
//	cbus_force_rcp();	
//}

static void mhltx_switch_work_func(struct work_struct *work) 
{

	int state =0,i;
	mhltx_device_t *pdev = container_of(sdev, mhltx_device_t,sdev);
	
	state = s_data.state;
	//MHL_DEBUG("[%s] %s  %d \n", __FILE__,__FUNCTION__,__LINE__);
	if(state == 1)
	{
		for(i=0; i<3; i++){
		
			CBUS_get_device_capability();
			
			MHL_INFO("CBUS_get_device_capability %d time(s)\n",i);
			
			if(rtk_adap -> msc_xfer.res_cmd == MHL_ACK)
				break;				
		}
		CBUS_dump_device_capability(&rtk_adap->rtk_mhltx_config.devcap);
		hdmitx_get_sink_capability((asoc_hdmi_t*)mhltx_get_drvdata(pdev));
	}
	else
	{
		CBUS_reset_device_capability(&rtk_adap->rtk_mhltx_config.devcap);
		hdmitx_reset_sink_capability((asoc_hdmi_t*)mhltx_get_drvdata(pdev));
	}

	switch_set_state(sdev, state);
					                      
}

static irqreturn_t mhltx_switch_isr(int irq, void *data)
{    				
	//MHL_DEBUG("[%s] cable %s \n", __FUNCTION__,(state==1)?"pluged in":"pulled out");
	
	if(gpio_get_value(s_data.pin)== plug_in_val)
		cbus_discovery_detect_1K();
	else
		cbus_HPD_handler(HPD_PULL_OUT,0); // doesn't trigger det-1K
    
	return IRQ_HANDLED;
}

static const struct of_device_id rtk_mhltx_hpd_dt_ids[] = {
     { .compatible = "Realtek,rtk119x-mhltx-hpd", },
     {},
};
MODULE_DEVICE_TABLE(of, rtk_mhltx_hpd_dt_ids);

int register_mhltx_switchdev(mhltx_device_t * device)
{
	int ret;
	static struct workqueue_struct *workq;
	
	struct device_node __maybe_unused *mhltx_hdp_node;
	unsigned int module_en = 0;
	int n,gpio;

			
	MHL_DEBUG("register_mhltx_switch \n");	
	
	sdev = kzalloc(sizeof(struct switch_dev), GFP_KERNEL);
	if (!sdev)
		return -ENOMEM;
		
	sdev = &device->sdev;		
 
	INIT_WORK(&s_data.work, mhltx_switch_work_func);
	
	sdev->name = MHL_SWITCH_NAME;
	sdev->print_state = mhltx_switch_print_state;

	ret = switch_dev_register(sdev);
	if (ret < 0) {
		MHL_ERROR("err_register_switch\n");
		goto err_register_switch;
	}

#if 1	
	mhltx_hdp_node = of_find_matching_node(NULL, rtk_mhltx_hpd_dt_ids);	
	of_property_read_u32(mhltx_hdp_node, "module-enable", &(module_en));
	of_property_read_u32(mhltx_hdp_node, "plug-in", &(plug_in_val));
	
	if (module_en) 
	{		
		MHL_INFO("Cable-HPD ok,plug in=%x\n",plug_in_val);
	
		CLEARBITS( 0xfe000000, 0x384 ,0x00200000);
	
		/*initial gpios for Hotplug*/	
		gpio = of_gpio_count(mhltx_hdp_node);
		
		if (gpio)
		{
			for (n = 0; n < gpio; n++) {
				s_data.pin = of_get_gpio_flags(mhltx_hdp_node, n, NULL); 	// get gpio number from device tree
				
				if (s_data.pin < 0) {
					MHL_ERROR("[%s ] could not get gpio from of \n",__FUNCTION__);
					//return -ENODEV;
				}
			
				if (!gpio_is_valid(s_data.pin))//check whether gpio number is valid
				{
					MHL_ERROR("[%s ] gpio %d is not valid\n",__FUNCTION__,s_data.pin);
					//return -ENODEV;
				}
			
				if(gpio_request(s_data.pin, mhltx_hdp_node->name))//request gpio			
				{
					MHL_ERROR("[%s ] could not request gpio, %d\n",__FUNCTION__, s_data.pin);
					//return -ENODEV;
				}
			}
		}
		
		gpio_direction_input(s_data.pin);	
		
		s_data.irq =  irq_of_parse_and_map (mhltx_hdp_node, 0);
			
		irq_set_irq_type(s_data.irq, IRQ_TYPE_EDGE_BOTH);
		
		if(request_irq(s_data.irq, mhltx_switch_isr,IRQF_SHARED,"switch_mhltx",&device->sdev)) {
			MHL_ERROR("cannot register IRQ %d\n", s_data.irq);
		}
		
		//INIT_WORK(&tv_work, mhltx_switch_tv_work_func);
	}
	else
		MHL_ERROR("Cable-HPD is disabled\n");
#endif

#if 1
	mhltx_switchdev_set_HPD_status(HPD_PLUG_IN);
	
	workq = create_singlethread_workqueue("switch-mhltx");
    if (!workq){
		MHL_ERROR("switch init work failed\n");
		//return -ENOMEM;		
	}
        
	queue_work(workq, &s_data.work);
 
	destroy_workqueue(workq);
#endif			
	goto end;
		

err_register_switch:	
	MHL_ERROR("register_mhltx_switch failed\n");
end:	
	return ret;
}
EXPORT_SYMBOL(register_mhltx_switchdev);


void deregister_mhltx_switchdev(mhltx_device_t* device)
{
    return switch_dev_unregister(&device-> sdev);
}
EXPORT_SYMBOL(deregister_mhltx_switchdev);


void mhltx_switchdev_HPD_handler(void)
{
#if 0	
	static struct workqueue_struct *workq;
	
	workq = create_singlethread_workqueue("switch-mhltx");
    if (!workq){
		MHL_ERROR("switch init work failed\n");
		return -ENOMEM;		
	}
        
	queue_work(workq, &s_data.work);
 
	destroy_workqueue(workq);   
#endif	

	schedule_work(&s_data.work); //schedule_work(&tv_work);

}
EXPORT_SYMBOL(mhltx_switchdev_HPD_handler);

int mhltx_switchdev_set_HPD_status(int HPD_status)
{		
	return s_data.state = HPD_status;
}
EXPORT_SYMBOL(mhltx_switchdev_set_HPD_status);


int mhltx_switchdev_show_HPD_status(void)
{		
	return s_data.state ;
}
EXPORT_SYMBOL(mhltx_switchdev_show_HPD_status);




