/*
 * Realtek MMC/SD/SDIO driver
 *
 * Authors:
 * Copyright (C) 2008-2013 Realtek Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/io.h>
#include <linux/platform_device.h>
#include <linux/mbus.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/dma-mapping.h>
#include <linux/scatterlist.h>
#include <linux/irq.h>
#include <linux/gpio.h>
#include <linux/mmc/host.h>
#include <asm/unaligned.h>

#include <linux/sched.h>                //liao
#include <linux/wait.h>                 //liao
#include <linux/slab.h>                 //liao
#include <linux/semaphore.h>           //liao
#include <linux/mmc/card.h>             //liao
#include <linux/mmc/mmc.h>              //liao
#include <linux/mmc/sd.h>               //liao
#include <linux/workqueue.h>            //liao
#include <linux/completion.h>           //liao
#include <rbus/reg_mmc.h>               //liao
#include <rbus/reg_iso.h>               //liao
#include "rtkemmc.h"                    //liao
#include "rtksd_ops.h"                    //liao
#include "../mmc_debug.h"               //liao

#ifdef MMC_DBG
static unsigned sd_reg = 0;
#endif
extern volatile unsigned char g_cmd[];

DECLARE_COMPLETION(rtk_emmc_wait);

#ifdef GPIO_DEBUG
void trans_db_gpio(void)
{
    //set debug GPIO
    u32 reginfo;
    //1. 0xB800_0804[31:28] = F    --> pin share as gpio
    //2. 0x1801_BC00[4] = 1  --> output mode
    //3. 0x1801_BC18[4]   is output data
    cr_writel(cr_readl(GP0DIR_reg)|0x10,GP0DIR_reg);

    reginfo = cr_readl(GP0DATO_reg);
    if(reginfo & 0x10){
        mmcrtk("GP HI\n");
        cr_writel(reginfo & ~0x10,GP0DATO_reg);
    }else{
        mmcrtk("GP LO\n");
        cr_writel(reginfo | 0x10,GP0DATO_reg);
    }

}

void trans_rst_gpio(void)
{
    //set rst GPIO
    u32 reginfo;
    cr_writel(cr_readl(GP0DIR_reg) |  0x00100000, GP0DIR_reg);
    reginfo = cr_readl(GP0DATO_reg);

    if(reginfo & 0x00100000){
        cr_writel(reginfo & ~0x00100000, GP0DATO_reg);
    }else{
        cr_writel(reginfo |  0x00100000, GP0DATO_reg);
    }

}
#else
#define trans_db_gpio()
#define trans_rst_gpio()
#endif

void sync(void)
{
    asm volatile("DMB");
    cr_writel(0x0, 0x1801a020);
    asm volatile("DMB");
}

EXPORT_SYMBOL_GPL(sync);

//#define INT_BLOCK_R_GAP 0x200
//#define INT_BLOCK_W_GAP 5
int rtkcr_wait_opt_end(char* drv_name, struct rtksd_host *sdport,u8 cmdcode,u8 cpu_mode)
{
    volatile u8 sd_transfer_reg;
    volatile int loops;
    volatile int err=CR_TRANS_OK;
    volatile unsigned long timeend=0;
    volatile unsigned int sd_trans=0;
    unsigned long flags;

    if (g_cmd[0] == 0x41)
    {
     loops = 10;  
     rtk_int_enable_and_waitfor(sdport,cmdcode,300,0);
     while(loops ){
            if (sdport->sd_trans & ERR_STATUS) //transfer error
            {
                printk(KERN_INFO "\n%s - trans error 2 :\ncmd : 0x%02x, trans : 0x%08x, st1 : 0x%08x, st2 : 0x%08x, bus : 0x%08x\n", drv_name, g_cmd[0], cr_readb(sdport->base_io+SD_TRANSFER), cr_readb(sdport->base_io+SD_STATUS1), cr_readb(sdport->base_io+SD_STATUS2), cr_readb(sdport->base_io+SD_BUS_STATUS));
                
                return CR_TRANSFER_FAIL;
            }
            if ((sdport->sd_trans & (END_STATE|IDLE_STATE))==(END_STATE|IDLE_STATE))
            {
                MMCPRINTF("\ncard transferred \n");
                
                err = CR_TRANS_OK;
                break;
            }
        //}
        
        //card busy ??
        if ((cr_readb(sdport->base_io+SD_CMD1)&0x80)!=0x80)
        {
            //resend cmd again
            cr_writeb(g_cmd[0], sdport->base_io+SD_CMD0);
            cr_writeb(g_cmd[1], sdport->base_io+SD_CMD1);
            cr_writeb(g_cmd[2], sdport->base_io+SD_CMD2);
            cr_writeb(g_cmd[3], sdport->base_io+SD_CMD3);
            cr_writeb(g_cmd[4], sdport->base_io+SD_CMD4);
	    //TODO : fix the complete object doesn't handle well at rtk_int_waitfor()
            //rtk_int_waitfor(sdport,300,0);
            rtk_int_enable_and_waitfor(sdport,cmdcode,300,0);

            printk(KERN_INFO "\n%s : card busy, retry cmd = 0x%08x\n", drv_name, g_cmd[0]);
        }
        else
            break;
      
        loops--;
     }
    }
    else if (g_cmd[0] == 0x42)
    {        
        if (cpu_mode == 0)
        {
            MMCPRINTF("\n%s - info :\ncmd0 : 0x%02x,cmd1 : 0x%02x,cmd2 : 0x%02x,cmd3 : 0x%02x,cmd4: 0x%02x\n", drv_name, g_cmd[0],g_cmd[1],g_cmd[2],g_cmd[3],g_cmd[4]);
            err = CR_DMA_FAIL;    
            rtk_int_enable_and_waitfor(sdport,cmdcode,500,3000);
            sync();
            MMCPRINTF("\n%s - info1 :\ncmd0 : 0x%02x,cmd1 : 0x%02x,cmd2 : 0x%02x,cmd3 : 0x%02x,cmd4: 0x%02x\n", drv_name, g_cmd[0],g_cmd[1],g_cmd[2],g_cmd[3],g_cmd[4]);
            timeend = jiffies + msecs_to_jiffies(3000);
            while (time_before(jiffies, timeend))
            {
                if (!(cr_readl(sdport->base_io+EMMC_DMA_CTL3) & DMA_XFER))
                    return CR_TRANS_OK;
		else
            	    err = CR_DMA_FAIL;    
            	cr_writeb(g_cmd[0], sdport->base_io+SD_CMD0);
            	cr_writeb(g_cmd[1], sdport->base_io+SD_CMD1);
            	cr_writeb(g_cmd[2], sdport->base_io+SD_CMD2);
            	cr_writeb(g_cmd[3], sdport->base_io+SD_CMD3);
            	cr_writeb(g_cmd[4], sdport->base_io+SD_CMD4);
                sync();
                rtk_int_enable_and_waitfor(sdport,cmdcode,500,500);
                //rtk_int_waitfor(sdport,cmdcode,300,300);
                sync();

            	MMCPRINTF("\n%s - info3 :\ncmd0 : 0x%02x,cmd1 : 0x%02x,cmd2 : 0x%02x,cmd3 : 0x%02x,cmd4: 0x%02x\n", drv_name, g_cmd[0],g_cmd[1],g_cmd[2],g_cmd[3],g_cmd[4]);
            }
            #if 1
            if (jiffies >=  timeend)
            #else
            if (err==CR_DMA_FAIL)
            #endif
            {
                printk(KERN_INFO "\n%s : cmd 2 wait DMA_BUF FULL to 1 fail. dma_ctl3 = 0x%08x\n", drv_name,cr_readl(sdport->base_io+CR_DMA_CTL3));
                return CR_BUF_FULL_TO;
            }
        }
        
        //polling the buf in 
        timeend = jiffies + msecs_to_jiffies(1000);
        while(time_before(jiffies, timeend))
        {
            if (cr_readl(sdport->base_io+EMMC_CPU_ACC) & BUF_FULL)
                break;
        }

        if (jiffies >=  timeend)
        {
            printk(KERN_INFO "\n%s : TO1, cmd 2 wait BUF_FULL to 1 fail. cpu_acc = 0x%08x\n", drv_name, cr_readl(sdport->base_io+EMMC_CPU_ACC));
            return CR_BUF_FULL_TO;
        }

        //polling the buf_full to 0
        cr_writel(0x3, sdport->base_io+EMMC_CPU_ACC);
        timeend = jiffies + msecs_to_jiffies(2);
        while(time_before(jiffies, timeend));
        sync();
        timeend = jiffies + msecs_to_jiffies(2000);
        while(time_before(jiffies, timeend))
        {
            if ((cr_readl(sdport->base_io+EMMC_CPU_ACC) & BUF_FULL) == 0x00)
                break;
        }
        if (jiffies >=  timeend)
        {
            printk(KERN_INFO "\n%s : TO2, cmd 2 wait BUF_FULL to 1 fail. 0x%08x\n", drv_name, cr_readl(sdport->base_io+EMMC_CPU_ACC));
            return CR_TRANSFER_TO;
        }
        
        cr_writel(0, sdport->base_io+CR_DMA_CTL3);

        MMCPRINTF("\ncmd 2 poll cpu_acc/dma to 0x%08x, 0x%08x\n", cr_readl(sdport->base_io+EMMC_CPU_ACC), cr_readl(sdport->base_io+CR_DMA_CTL3));
    }
    else
    {
        err = CR_TRANSFER_TO;
	sync();
        MMCPRINTF("\n%s - info4 :\ncmd0 : 0x%02x,cmd1 : 0x%02x,cmd2 : 0x%02x,cmd3 : 0x%02x,cmd4: 0x%02x\n", drv_name, g_cmd[0],g_cmd[1],g_cmd[2],g_cmd[3],g_cmd[4]);
        rtk_int_enable_and_waitfor(sdport,cmdcode,6000,0);
        {
	    sd_trans = sdport->sd_trans;
            sync();
            if ((sd_trans & (END_STATE|IDLE_STATE))==(END_STATE|IDLE_STATE))
            {
        	    err = CR_TRANS_OK;
        		//break;
       	    }
            if ((sd_trans & (ERR_STATUS))==(ERR_STATUS))
            {
        	    err = CR_TRANSFER_FAIL;
            	    printk(KERN_INFO "\n%s - trans error 5 :\ncmd : 0x%02x, cfg1 : 0x%02x, cfg2 : 0x%02x, cfg3 : 0x%02x, trans : 0x%08x, st1 : 0x%08x, st2 : 0x%08x, bus : 0x%08x\n", drv_name, g_cmd[0],cr_readb(sdport->base_io+SD_CONFIGURE1),cr_readb(sdport->base_io+SD_CONFIGURE2),cr_readb(sdport->base_io+SD_CONFIGURE3), cr_readb(sdport->base_io+SD_TRANSFER), cr_readb(sdport->base_io+SD_STATUS1), cr_readb(sdport->base_io+SD_STATUS2), cr_readb(sdport->base_io+SD_BUS_STATUS));
        		//break;
            }
        }
        if (err == CR_TRANSFER_TO)
        {
            printk(KERN_INFO "\n%s - trans error 3 :\ncmd : 0x%02x, trans : 0x%08x, st1 : 0x%08x, st2 : 0x%08x, bus : 0x%08x\n", drv_name, g_cmd[0], cr_readb(sdport->base_io+SD_TRANSFER), cr_readb(sdport->base_io+SD_STATUS1), cr_readb(sdport->base_io+SD_STATUS2), cr_readb(sdport->base_io+SD_BUS_STATUS));
            return err;
        }            

        if (g_cmd[0] == 0x49)
        {
          err = CR_DMA_FAIL;

          timeend = jiffies + msecs_to_jiffies(3000);
          while(time_before(jiffies, timeend))
          {
            if (!(cr_readl(sdport->base_io+EMMC_DMA_CTL3) & DMA_XFER))
            {
                err = CR_TRANS_OK;
                break;
            }
          }
          #if 1
          if (jiffies >=  timeend)
          #else
          if (err==CR_DMA_FAIL)
          #endif
          {
                printk(KERN_INFO "%s : TO1, cmd 9 poll dma_done timeout : dma_ctl3 = 0x%08x\n", drv_name, cr_readl(sdport->base_io+CR_DMA_CTL3));
                return CR_TRANSFER_TO; 
          }
        }

    }

    if (cpu_mode == 1)
    {
      err = CR_BUF_FULL_TO;
      MMCPRINTF("\ncr cpu mode - read\n");
      timeend = jiffies + msecs_to_jiffies(3000);
      while(time_before(jiffies, timeend))
      {
    	if ((cr_readl(sdport->base_io+EMMC_CPU_ACC) & BUF_FULL) == BUF_FULL)
        {
    	    return CR_TRANS_OK;
    	}
      }
      return CR_DMA_FAIL;
    }

    return err;
}
EXPORT_SYMBOL_GPL(rtkcr_wait_opt_end);

void rtk_int_enable_and_waitfor(struct rtksd_host *sdport, u8 cmdcode, unsigned long msec, unsigned long dma_msec)
{
    unsigned long timeend=0;

    sdport->int_status  = 0;
    sdport->sd_trans    = -1;
    sdport->sd_status1   = 0;
    sdport->sd_status2   = 0;
    sdport->bus_status   = 0;
    sdport->dma_trans   = 0;

    #ifndef ENABLE_EMMC_INT_MODE
    sdport->int_waiting = NULL;
    #else
    sdport->int_waiting = &rtk_emmc_wait;
    MMCPRINTF("rtk wait complete addr = %08x\n", (unsigned int) sdport->int_waiting);
    /* timeout timer fire */
    if (&sdport->timer)
    {
        MMCPRINTF("timer started : ");
        #if 0
        if(sdport->tmout){
            timeend = sdport->tmout;
            sdport->tmout = 0;
        }else
        #endif
            timeend = msecs_to_jiffies(msec)+sdport->tmout;

        MMCPRINTF("TO = 0x%08x\n", timeend);
        mod_timer(&sdport->timer, (jiffies + timeend) );
    }

    //wait for 
    #endif
    rtk_int_waitfor(sdport,cmdcode,msec,dma_msec);
    smp_wmb();
}
EXPORT_SYMBOL_GPL(rtk_int_enable_and_waitfor);

void rtk_int_waitfor(struct rtksd_host *sdport, u8 cmdcode, unsigned long msec,unsigned long dma_msec)
{
    #ifndef ENABLE_EMMC_INT_MODE
    unsigned long timeend=0;
    u32 sd_trans=0,sd_status1=0,sd_status2=0,bus_status=0,dma_trans=0;
    unsigned long flags;

    spin_lock_irqsave(&sdport->lock,flags);

    if(sdport->rtflags & RTKCR_FOPEN_LOG){
        printk(" rtkemmc : register settings(base=0x%08x,0x%08x)\n", sdport->base_io, sdport->base_io+SD_CMD0);
        printk(" cmd0:0x%02x cmd1:0x%02x cmd2:0x%02x cmd3:0x%02x cmd4:0x%02x cmd5:0x%02x\n",cr_readb(sdport->base_io+SD_CMD0),cr_readb(sdport->base_io+SD_CMD1),cr_readb(sdport->base_io+SD_CMD2),cr_readb(sdport->base_io+SD_CMD3),cr_readb(sdport->base_io+SD_CMD4),cr_readb(sdport->base_io+SD_CMD5));
        printk(" trans:0x%02x status1:0x%02x status2:0x%02x bus_status:0x%02x\n",cr_readb(sdport->base_io+SD_TRANSFER),cr_readb(sdport->base_io+SD_STATUS1),cr_readb(sdport->base_io+SD_STATUS2),cr_readb(sdport->base_io+SD_BUS_STATUS));
        printk(" configure1:0x%02x configure2:0x%02x configure3:0x%02x\n",cr_readb(sdport->base_io+SD_CONFIGURE1),cr_readb(sdport->base_io+SD_CONFIGURE2),cr_readb(sdport->base_io+SD_CONFIGURE3));
        printk(" byteH:0x%02x byteL:0x%02x blkH:0x%02x blkL:0x%02x\n",cr_readb(sdport->base_io+SD_BYTE_CNT_H),cr_readb(sdport->base_io+SD_BYTE_CNT_L),cr_readb(sdport->base_io+SD_BLOCK_CNT_H),cr_readb(sdport->base_io+SD_BLOCK_CNT_L));
        printk(" CPU_ACC:0x%08x dma_ctl1:0x%08x dma_ctl2:0x%08x dma_ctl3:0x%08x\n",cr_readl(sdport->base_io+EMMC_CPU_ACC),cr_readl(sdport->base_io+CR_DMA_CTL1),cr_readl(sdport->base_io+CR_DMA_CTL2),cr_readl(sdport->base_io+CR_DMA_CTL3));
    }

    //cmd fire
    #ifdef MMC_DBG
    sd_reg = cr_readb(sdport->base_io+SD_CMD0);    
    #endif
    sync();
    cr_writeb((u8) (cmdcode|START_EN), sdport->base_io+SD_TRANSFER );
    sync();
    
    timeend = jiffies + msecs_to_jiffies(msec);
    while(time_before(jiffies, timeend))
    {
        if ((cr_readb(sdport->base_io+SD_TRANSFER) & (END_STATE|IDLE_STATE))==(END_STATE|IDLE_STATE))
            break;
        if ((cr_readb(sdport->base_io+SD_TRANSFER) & ERR_STATUS)==(ERR_STATUS))
            break;        
    }

    if (dma_msec>0)
    {
        timeend = jiffies + msecs_to_jiffies(dma_msec);
        while(time_before(jiffies, timeend))
        {
            if ((cr_readb(sdport->base_io+CR_DMA_CTL3) & DMA_XFER)!=(DMA_XFER))
                break;
        }        
        rtkcr_get_dma_trans(sdport->base_io,&dma_trans);
        sdport->dma_trans    = dma_trans;
    }
    MMCPRINTF("exit from polling\n");

    rtkcr_get_sd_trans(sdport->base_io,&sd_trans);
    rtkcr_get_sd_sta(sdport->base_io,&sd_status1,&sd_status2,&bus_status);

    sdport->sd_trans    = sd_trans;
    sdport->sd_status1   = sd_status1;
    sdport->sd_status2   = sd_status2;
    sdport->bus_status   = bus_status;
    MMCPRINTF("int sts : 0x%08x sd_trans : 0x%08x, sd_st1 : 0x%08x\n", sdport->int_status, sdport->sd_trans, sdport->sd_status1);
    MMCPRINTF("int st2 : 0x%08x bus_sts : 0x%08x dma_trans : 0x%08x\n", sdport->sd_status2, sdport->bus_status, sdport->dma_trans);
    spin_unlock_irqrestore(&sdport->lock,flags);
    #else
    sync();
    if(sdport->rtflags & RTKCR_FOPEN_LOG){
        printk(" rtkemmc : register settings(base=0x%08x,0x%08x)\n", sdport->base_io, sdport->base_io+SD_CMD0);
        printk(" cmd0:0x%02x cmd1:0x%02x cmd2:0x%02x cmd3:0x%02x cmd4:0x%02x cmd5:0x%02x\n",cr_readb(sdport->base_io+SD_CMD0),cr_readb(sdport->base_io+SD_CMD1),cr_readb(sdport->base_io+SD_CMD2),cr_readb(sdport->base_io+SD_CMD3),cr_readb(sdport->base_io+SD_CMD4),cr_readb(sdport->base_io+SD_CMD5));
        printk(" trans:0x%02x status1:0x%02x status2:0x%02x bus_status:0x%02x\n",cr_readb(sdport->base_io+SD_TRANSFER),cr_readb(sdport->base_io+SD_STATUS1),cr_readb(sdport->base_io+SD_STATUS2),cr_readb(sdport->base_io+SD_BUS_STATUS));
        printk(" configure1:0x%02x configure2:0x%02x configure3:0x%02x\n",cr_readb(sdport->base_io+SD_CONFIGURE1),cr_readb(sdport->base_io+SD_CONFIGURE2),cr_readb(sdport->base_io+SD_CONFIGURE3));
        printk(" byteH:0x%02x byteL:0x%02x blkH:0x%02x blkL:0x%02x\n",cr_readb(sdport->base_io+SD_BYTE_CNT_H),cr_readb(sdport->base_io+SD_BYTE_CNT_L),cr_readb(sdport->base_io+SD_BLOCK_CNT_H),cr_readb(sdport->base_io+SD_BLOCK_CNT_L));
        printk(" CPU_ACC:0x%08x dma_ctl1:0x%08x dma_ctl2:0x%08x dma_ctl3:0x%08x\n",cr_readl(sdport->base_io+EMMC_CPU_ACC),cr_readl(sdport->base_io+CR_DMA_CTL1),cr_readl(sdport->base_io+CR_DMA_CTL2),cr_readl(sdport->base_io+CR_DMA_CTL3));
    }
    
    if (sdport->int_waiting)
    {
        #ifdef MMC_DBG
        sd_reg = cr_readb(sdport->base_io+SD_CMD0);
        MMCPRINTF("========== S(0x%02x) ==========\n", sd_reg);
        #endif
        rtkcr_hold_int_dec(sdport->base_io);
        rtkcr_clr_int_sta(sdport->base_io);
        rtkcr_en_int(sdport->base_io);
        rtkcr_clr_int_sta(sdport->base_io);

        sync();
        //cmd fire
        cr_writeb((u8) (cmdcode|START_EN), sdport->base_io+SD_TRANSFER );
        sync();    
        
        MMCPRINTF("do wait for 1\n");
        wait_for_completion(sdport->int_waiting);
        MMCPRINTF("do wait for 2\n");
    }
    #endif
}
EXPORT_SYMBOL_GPL(rtk_int_waitfor);

void rtk_op_complete(struct rtksd_host *sdport)
{
    if (sdport->int_waiting) {
        MMCPRINTF("int wait complete 1\n");
        MMCPRINTF("1 rtk wait complete addr = %08x\n", (unsigned int) sdport->int_waiting);
        struct completion *waiting = sdport->int_waiting;
        //sdport->int_waiting = NULL;
        complete(waiting);
        MMCPRINTF("========== D ==========\n");
        sync();
        MMCPRINTF("int wait complete 2\n");
    }
    else
        MMCPRINTF("int wait not complete\n");
}
EXPORT_SYMBOL_GPL(rtk_op_complete);

char *rtkcr_parse_token(const char *parsed_string, const char *token)
{
    const char *ptr = parsed_string;
    const char *start, *end, *value_start, *value_end;
    char *ret_str;

    while(1) {
        value_start = value_end = 0;
        for(;*ptr == ' ' || *ptr == '\t'; ptr++);
        if(*ptr == '\0')        break;
        start = ptr;
        for(;*ptr != ' ' && *ptr != '\t' && *ptr != '=' && *ptr != '\0'; ptr++) ;
        end = ptr;
        if(*ptr == '=') {
            ptr++;
            if(*ptr == '"') {
                ptr++;
                value_start = ptr;
                for(; *ptr != '"' && *ptr != '\0'; ptr++);
                if(*ptr != '"' || (*(ptr+1) != '\0' && *(ptr+1) != ' ' && *(ptr+1) != '\t')) {
                    printk("system_parameters error! Check your parameters     .");
                    break;
                }
            } else {
                value_start = ptr;
                for(;*ptr != ' ' && *ptr != '\t' && *ptr != '\0' && *ptr != '"'; ptr++) ;
                if(*ptr == '"') {
                    printk("system_parameters error! Check your parameters.");
                    break;
                }
            }
            value_end = ptr;
        }

        if(!strncmp(token, start, end-start)) {
            if(value_start) {
                ret_str = kmalloc(value_end-value_start+1, GFP_KERNEL);
                // KWarning: checked ok by alexkh@realtek.com
                if(ret_str){
                    strncpy(ret_str, value_start, value_end-value_start);
                    ret_str[value_end-value_start] = '\0';
                }
                return ret_str;
            } else {
                ret_str = kmalloc(1, GFP_KERNEL);
                // KWarning: checked ok by alexkh@realtek.com
                if(ret_str)
                    strcpy(ret_str, "");
                return ret_str;
            }
        }
    }

    return (char*)NULL;
}
EXPORT_SYMBOL_GPL(rtkcr_parse_token);

void rtkcr_chk_param(u32 *pparam, u32 len, u8 *ptr)
{
    u32 value,i;
    mmcrtk("\n");

    *pparam = 0;
    for(i=0;i<len;i++){
        value = ptr[i] - '0';
        // KWarning: checked ok by alexkh@realtek.com
        if((value >= 0) && (value <=9))
        {
            *pparam+=value<<(4*(len-1-i));
            continue;
        }

        value = ptr[i] - 'a';
        // KWarning: checked ok by alexkh@realtek.com
        if((value >= 0) && (value <=5))
        {
            value+=10;
            *pparam+=value<<(4*(len-1-i));
            continue;
        }

        value = ptr[i] - 'A';
        // KWarning: checked ok by alexkh@realtek.com
        if((value >= 0) && (value <=5))
        {
            value+=10;
            *pparam+=value<<(4*(len-1-i));
            continue;
        }
    }
}
EXPORT_SYMBOL_GPL(rtkcr_chk_param);

u32 verA_magic_num = 0;
int rtkcr_chk_VerA(void)
{
    mmcrtk("\n");
//TODO : chk
#if 0 
    if(!verA_magic_num)
        verA_magic_num = cr_readl(0xb8060000);

    if(verA_magic_num == 0x62270000){
        return 1;
    }else{
        return 0;
    }
#else
    return verA_magic_num; //0: polling, 1: interrupt
#endif
}
EXPORT_SYMBOL_GPL(rtkcr_chk_VerA);

void emmc_show_config123(struct rtksd_host *sdport){
    u32 reginfo;
    u32 clksel_sht;
    u32 iobase = sdport->base_io;

    if(iobase == EM_BASE_ADDR){
        reginfo = cr_readl(iobase+EMMC_CKGEN_CTL);
        //clksel_sht = EMMC_CLKSEL_SHT;
    }else if(iobase == CR_BASE_ADDR){
        reginfo = cr_readl(iobase+CR_SD_CKGEN_CTL);
        //clksel_sht = SD_CLKSEL_SHT;
    }

    printk("CFG1=0x%x CFG2=0x%x CFG3=0x%x bus clock CKGEN=%08x\n",
        cr_readl(iobase+SD_CONFIGURE1),
        cr_readl(iobase+SD_CONFIGURE2),
        cr_readl(iobase+SD_CONFIGURE3),
        reginfo );
}
EXPORT_SYMBOL_GPL(emmc_show_config123);

#if 0//def CONFIG_REALTEK_GPIO	mask by victor for compile fail with this config  20140128

void rtkcr_set_mis_gpio(u32 gpio_num,u8 dir,u8 level)
{
    u32 reginfo = 0;
    u32 reg_tmp = 0;
    u32 bit_tmp = 0;

    if(gpio_num > MAX_MIS_GPIO_CNT){
        WARN_ON(1);
        return;
    }else{
        reg_tmp = ((gpio_num & 0xffffffe0)>>5)<<2;
        bit_tmp = (gpio_num & 0x1fUL);
    }

    /* Set data */
    if(dir == GPIO_OUT){
        if(level == GPIO_HIGH)  //GP0DATO_reg
            reginfo = cr_readl(GP0DATO_reg+reg_tmp) | (0x01UL<<bit_tmp);
        else
            reginfo = cr_readl(GP0DATO_reg+reg_tmp) & ~(0x01UL<<bit_tmp);

        cr_writel(reginfo,GP0DATO_reg+reg_tmp);

        mmcinfo(KERN_ERR "0x%x = 0x%08x\n",
                GP0DATO_reg+reg_tmp, cr_readl(GP0DATO_reg+reg_tmp));
    }

    /* Set dir */
    if(dir == GPIO_OUT)
        reginfo = cr_readl(GP0DIR_reg+reg_tmp) | (0x01UL<<bit_tmp);
    else
        reginfo = cr_readl(GP0DIR_reg+reg_tmp) & ~(0x01UL<<bit_tmp);
    cr_writel(reginfo,GP0DIR_reg+reg_tmp);
    mmcinfo(KERN_ERR "0x%x = 0x%08x\n",
                GP0DIR_reg+reg_tmp, cr_readl(GP0DIR_reg+reg_tmp));

    /* Set pin share */
    reg_tmp = (gpio_num & 0xfffffffc);
    bit_tmp = (gpio_num & 0x03UL);

    reginfo = cr_readl(GPIO_MUXCFG_0+reg_tmp)
                | (0xfUL << mis_gpio_bit_map[bit_tmp]);
    cr_writel(reginfo,GPIO_MUXCFG_0+reg_tmp);
    mmcinfo(KERN_ERR "0x%x = 0x%08x\n",
                GPIO_MUXCFG_0+reg_tmp,
                cr_readl(GPIO_MUXCFG_0+reg_tmp));
}
EXPORT_SYMBOL_GPL(rtkcr_set_mis_gpio);

void rtkcr_set_iso_gpio(u32 gpio_num,u8 dir,u8 level)
{
    u32 reginfo = 0;
    u32 reg_tmp = 0;
    u32 bit_tmp = 0;

    if( gpio_num > MAX_ISO_GPIO_CNT )
    {
        WARN_ON(1);
        return;
    }else{
        if(st_gpio_map[gpio_num] == 0xff){
            WARN_ON(1);
            return;
        }
        reg_tmp = ((gpio_num & 0xffffffe0)>>5)<<2;
        bit_tmp = (gpio_num & 0x1fUL);
    }

    /* Set data */
    if(dir == GPIO_OUT){
        if(level == GPIO_HIGH)
            reginfo = cr_readl(ISO_GPDATO+reg_tmp) | (0x01UL<<bit_tmp);
        else
            reginfo = cr_readl(ISO_GPDATO+reg_tmp) & ~(0x01UL<<bit_tmp);

        cr_writel(reginfo,ISO_GPDATO+reg_tmp);

        printk(KERN_ERR "0x%x = 0x%x\n",
                ISO_GPDATO+reg_tmp, cr_readl(ISO_GPDATO+reg_tmp));
    }

    /* Set dir */
    if(dir == GPIO_OUT)
        reginfo = cr_readl(ISO_GPDIR+reg_tmp) | (0x01UL<<bit_tmp);
    else
        reginfo = cr_readl(ISO_GPDIR+reg_tmp) & ~(0x01UL<<bit_tmp);
    cr_writel(reginfo,ISO_GPDIR+reg_tmp);
    printk(KERN_ERR "0x%x = 0x%x\n",
                ISO_GPDIR+reg_tmp, cr_readl(ISO_GPDIR+reg_tmp));

    /* Set pin share */
    reginfo = cr_readl(ISO_ST_CFG_0+st_gpio_map[gpio_num])
                | (0xfUL << st_gpio_bit_map[gpio_num]);
    cr_writel(reginfo,ISO_ST_CFG_0+st_gpio_map[gpio_num]);
    printk(KERN_ERR "0x%x = 0x%x\n",
                ISO_ST_CFG_0+st_gpio_map[gpio_num],
                cr_readl(ISO_ST_CFG_0+st_gpio_map[gpio_num]));
}
EXPORT_SYMBOL_GPL(rtkcr_set_iso_gpio);
#endif
/* end of file */

